package ve.gob.cnti.gestion.bonita;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.HttpHostConnectException;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.PoolingClientConnectionManager;
import org.apache.http.impl.conn.SchemeRegistryFactory;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;

import ve.gob.cnti.gestion.utils.PropertiesReader;

public class BonitaApiRest {
	
	PropertiesReader pr = new PropertiesReader();
	
	private HttpClient httpClient;
	

	public void setHttpClient(HttpClient httpClient) {
		this.httpClient = httpClient;
	}

	private HttpContext httpContext;
	private String serverurl = pr.bonitaProperties("SERVER_URL");
	private String applicationname = pr.bonitaProperties("APPLICATION_NAME");
	private String bonitaURI = serverurl+"/"+applicationname;
	
	private String response;
	
	
	/**TODO
	 * Listar Procesos Desplegados
	 * 
	 * Crear bean para retornar lista de respuestas
	 * 
	 * Implementar paginación
	 * 
	 * implementar logueo de usuario
	 * 
	 * implemntar filtro
	 */
	public String listProcessDeployed() {		
		
		PoolingClientConnectionManager conMan = getConnectionManager();
		setHttpClient(new DefaultHttpClient(conMan));
		
		loginAsAdmin();
		makeSureLocaleIsActive();
		
		consumeResponse(executeGetRequest("/API/bpm/process?p=0&c=10"),true);
		
		logout();
		httpClient.getConnectionManager().shutdown();
		return this.response;
	}
	
	

	/** TODO
	 * 
	 * Conexion a bonita mediante servicios rest
	 */
	private static PoolingClientConnectionManager getConnectionManager() {
		PoolingClientConnectionManager conMan = new PoolingClientConnectionManager(SchemeRegistryFactory.createDefault());
		conMan.setMaxTotal(200);
		conMan.setDefaultMaxPerRoute(200);
		return conMan;
	}

	
	
	
	
	private HttpResponse executeGetRequest(String apiURI) {
		try {
			HttpGet getRequest = new HttpGet(bonitaURI + apiURI);

			HttpResponse response = httpClient.execute(getRequest, httpContext);

			return response;

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

	}
	
	public void loginAs(String username, String password) {

		try {

			CookieStore cookieStore = new BasicCookieStore();
			httpContext = new BasicHttpContext();
			httpContext.setAttribute(ClientContext.COOKIE_STORE, cookieStore);

			String loginURL = "/loginservice";

			// If you misspell a parameter you will get a HTTP 500 error
			List<NameValuePair> urlParameters = new ArrayList<NameValuePair>();
			urlParameters.add(new BasicNameValuePair("username", username));
			urlParameters.add(new BasicNameValuePair("password", password));
			urlParameters.add(new BasicNameValuePair("redirect", "false"));

			// UTF-8 is mandatory otherwise you get a NPE
			UrlEncodedFormEntity entity = new UrlEncodedFormEntity(urlParameters, "utf-8");
			executePostRequest(loginURL, entity);

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

	}
	
	public void logout() {
		consumeResponse(executeGetRequest("/logoutservice"),false);
	}
			
	private int executePostRequest(String apiURI, UrlEncodedFormEntity entity) {
		try {
			HttpPost postRequest = new HttpPost(bonitaURI + apiURI);

			postRequest.setEntity(entity);

			HttpResponse response = httpClient.execute(postRequest, httpContext);

			return consumeResponse(response, true);

		} catch (HttpHostConnectException e) {
			throw new RuntimeException("Bonita bundle may not have been started, or the URL is invalid. Please verify hostname and port number. URL used is: " + bonitaURI,e);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

	}
	
	/**TODO
	 * 
	 * Implementar una forma de consumir la respuesta en un bean
	 * 
	 * @param response
	 * @param printResponse
	 * @return
	 */
	private int consumeResponse(HttpResponse response, boolean printResponse) {

		String responseAsString = consumeResponseIfNecessary(response);
		if(printResponse) {			
			this.response = responseAsString;
		}

		return ensureStatusOk(response);
	}

	private String consumeResponseIfNecessary(HttpResponse response) {
		if (response.getEntity() != null) {
			BufferedReader rd;
			try {
				rd = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));

				StringBuffer result = new StringBuffer();
				String line = "";
				while ((line = rd.readLine()) != null) {
					result.append(line);
				}
				return result.toString();
			} catch (Exception e) {
				throw new RuntimeException("Failed to consume response.", e);
			}
		} else {
			return "";
		}
	}
	
	private int ensureStatusOk(HttpResponse response) {
		if (response.getStatusLine().getStatusCode() != 201 && response.getStatusLine().getStatusCode() != 200) {
			throw new RuntimeException("Failed : HTTP error code : " + response.getStatusLine().getStatusCode() + " : "
					+ response.getStatusLine().getReasonPhrase());
		}
		return response.getStatusLine().getStatusCode();
	}
	
	private void makeSureLocaleIsActive() {
		consumeResponse(executeGetRequest("/API/system/i18ntranslation?f=locale%3den"), false);
	}
	
	
	/**
	 * Autenticación de usuarios para pruebas
	 */
	private void loginAsAdmin() {
		loginAs(pr.bonitaProperties("ADMIN_USERNAME"), pr.bonitaProperties("ADMIN_PASSWORD"));
	}

	/**
	 * TODO
	 */
//	public int startACase(long processDefinitionId, long taskId, String userId ) {
//		String apiURI = "/API/bpm/case/";
//
//		String payloadAsString = "{\"processDefinitionId\": " + processDefinitionId + "}";
//
//		consumeResponse(executePostRequest(apiURI, payloadAsString),true);
//
//		consumeResponse(executeGetRequest("/API/bpm/case?p=0&c=1&f=user_id%3d" + userId),true);
//
//
//		String payload1AsString = "{\"assigned_id\":\"" + userId + "\"}";
//		consumeResponse(executePutRequest("/API/bpm/humanTask/" + taskId, payload1AsString),true);
//
//		return consumeResponse(executeGetRequest("/API/bpm/humanTask?p=0&c=100&f=state%3dready&f=user_id%3d" + userId),true);
//
//		}
}
