package ve.gob.cnti.gestion.bonita;

import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

import org.json.JSONException;

import ve.gob.cnti.gestion.bonita.bean.Task;

public interface ITask extends Serializable{
	
	/**
	 * TODO
	 * @param id
	 * @return
	 * @throws IOException 
	 * @throws JSONException 
	 * @throws IllegalStateException 
	 * @throws UnsupportedEncodingException 
	 * @throws InterruptedException 
	 */
	public Task getTaskbyCaseId(String idCase) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException, InterruptedException;
	
	/**
	 * 
	 * @param userId
	 * @param taskId
	 */
	public void assignTask(String userId, String taskId);

	/**
	 * 
	 * @param idTask
	 * @param payload
	 */
	//void updateTaskVariables(String idTask, String payload);
	void updateTaskVariables(String idTask, Map<String, Serializable> variables);
	/**
	 * 
	 * @param idTask
	 */
	void executeTask(String idTask);

	/**
	 * 
	 * @return
	 * @throws UnsupportedEncodingException
	 * @throws IllegalStateException
	 * @throws JSONException
	 * @throws IOException
	 */
	List<Task> getUnassignedTasks() throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException;

	/**
	 * 
	 * @param taskId
	 */
	void releaseTask(String taskId);
	
	/**
	 * Devuelve las tareas asignadas a un usuario
	 * @param userId
	 * @return
	 * @throws UnsupportedEncodingException
	 * @throws IllegalStateException
	 * @throws JSONException
	 * @throws IOException
	 */
	List<Task> getAssignedTasks(String userId)
			throws UnsupportedEncodingException, IllegalStateException,
			JSONException, IOException;

	/**
	 * @author José Francisco Luis Medina
	 * @param idTask
	 * @return retorna un objeto del tipo Task
	 * @throws IOException 
	 * @throws JSONException 
	 * @throws IllegalStateException 
	 * @throws UnsupportedEncodingException 
	 */
	Task getTask(String idTask) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException;

	
	/**
	 * Retrna una lista de tareas disponibles para ejecutar por un usuario
	 * @author José Francisco Luis Medina
	 * @param idUser
	 * @return
	 */
	List<Task> gesTasksAvailableForUser(String idUser);

	/**
	 * Retorna la tarea dependiente o de ejecucion de la tarea de asignacion indicada
	 * @author José Francisco Luis Medina
	 * @param masterTask
	 * @return
	 * @throws IOException 
	 * @throws UnsupportedEncodingException 
	 */
	String getDependingTask(String IdMasterTask) throws UnsupportedEncodingException, IOException;


	/**
	 * Retorna las tareas de asignación activas para un usuario determinado
	 * @author José Francisco Luis Medina
	 */
	List<Task> getAssignmentTasks(String idUser);

	

	/**
	 * IMPORTANTE!!
	 * 
	 * Asigna la tarea activa al usuario que inicio el proceso, este método solo puede ser 
	 * utilizado en el caso que este garantizado que solo habrá una sola tarea 
	 * activa sin ningun tipo de paralelismos
	 * 
	 * @author José Francisco Luis Medina
	 * @param idCase id de la instancia del proceso
	 * @param idTask ide de la tarea a asignar
	 */
	void assignToStarter(String idCase);
	
	
	//TODO documentar
	public List<Task> getAllHumanActivities(String idCase);
	
	/**
	 * IMPORTANTE!!
	 * 
	 * Este metodo tiene como finalidad trear la ultima tarea de los casos archivados
	 * 
	 * @author Yamil Orozco
	 * @param id es el id del caso
	 */
	public Task getTaskbyCaseIdHistory(String id);
	/**
	 * IMPORTANTE!!
	 * 
	 * Este metodo tiene como finalidad traer todas las tareas de los casos finalizados
	 * 
	 * @author Yamil Orozco
	 * @param id es el id del caso
	 */
	
	public List<Task> getTaskCompleted(String idCase);
}
