package ve.gob.cnti.gestion.bonitaImpl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.engine.bpm.data.DataInstance;
import org.bonitasoft.engine.bpm.data.DataNotFoundException;
import org.bonitasoft.engine.bpm.flownode.ArchivedActivityInstance;
import org.bonitasoft.engine.bpm.flownode.ArchivedHumanTaskInstanceSearchDescriptor;
import org.bonitasoft.engine.bpm.process.ArchivedProcessInstance;
import org.bonitasoft.engine.bpm.process.ArchivedProcessInstanceNotFoundException;
import org.bonitasoft.engine.bpm.process.ArchivedProcessInstancesSearchDescriptor;
import org.bonitasoft.engine.bpm.process.ProcessInstance;
import org.bonitasoft.engine.bpm.process.ProcessInstanceNotFoundException;
import org.bonitasoft.engine.bpm.process.ProcessInstanceSearchDescriptor;
import org.bonitasoft.engine.exception.SearchException;
import org.bonitasoft.engine.platform.LoginException;
import org.bonitasoft.engine.search.SearchOptionsBuilder;
import org.bonitasoft.engine.search.SearchResult;
import org.bonitasoft.engine.session.APISession; 

import ve.gob.cnti.gestion.bonita.ICase;
import ve.gob.cnti.gestion.bonita.bean.Case;
import ve.gob.cnti.gestion.bonita.bean.ConsultaGenerica;
import ve.gob.cnti.gestion.bonita.bean.ProcessDefinition;
import ve.gob.cnti.gestion.jdbc.JConnection;
import ve.gob.cnti.gestion.utils.FacesHandler;
import ve.gob.cnti.gestion.utils.JsonManager;

public class CaseImpl implements ICase{

	private SessionImpl sessionImpl = null;
	private JsonManager jManager = null;	
	private Case caseDef = null;
	private FacesHandler facehandler = null;
	
	public CaseImpl(){
		
		this.sessionImpl = new SessionImpl();
		this.jManager = new JsonManager();
		this.caseDef = new Case();
		this.facehandler = new FacesHandler();
		
	}
	
	/**
	 * Este método devuelve una instancia de un Caso
	 * @author José Francisco Luis Medina.
	 * @param idCase Recibe el id del caso a obtener
	 * @return caseObj 
	 */
	@Override
	public Case getCase(String idCase){
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		Case caseObj = null;
	
		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			ProcessInstance pi = processAPI.getProcessInstance(Long.parseLong(idCase));
			caseObj = beanCase2stringBeanCase(pi);
		} catch (ProcessInstanceNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return caseObj;
	}
	
	
	/**
	 * Este método devuelve una instancia de un Caso archivado
	 * @author José Francisco Luis Medina
	 * @param idCase Recibe el id del caso a obtener
	 * @return caseObj 
	 */
	@Override
	public Case getArchivedCase(String idCase){
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		Case caseObj = null;
	
		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			ArchivedProcessInstance api = processAPI.getArchivedProcessInstance(Long.parseLong(idCase));
			caseObj = beanArchivedCase2stringBeanCase(api);			
			
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ArchivedProcessInstanceNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return caseObj;
	}
	

	
	/**
	 * Este método recibe el id de un usuario y devuelve la lista de casos que ese usuario ha iniciado
	 * @author José Francisco Luis Medina
	 * @param idUser
	 * @return List<Case>
	 */
	@Override
	public List<Case> listCases(String idUser){
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		ArrayList<Case> cases = new ArrayList<Case>();

		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 10000);
			builder.filter(ProcessInstanceSearchDescriptor.STARTED_BY, idUser);
			SearchResult<ProcessInstance> pi = processAPI.searchProcessInstances(builder.done());
			
			for (ProcessInstance procInst : pi.getResult()) {
				cases.add(bonitaProcInst2StringCaseBean(procInst));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return cases;
	}
	
	/**
	 * Recibe un objeto del tipo ProcessInstance natural del engine de Bonita y lo convierte en un objeto
	 * del tipo Case definido como bean en el proyecto
	 * 
	 * @author José Francisco Luis Medina
	 * @param pi
	 * @return
	 */
	private Case bonitaProcInst2StringCaseBean(ProcessInstance pi){	
		Case caseObj = new Case(
				String.valueOf(pi.getId()),
				String.valueOf(pi.getEndDate()),
				String.valueOf(pi.getStartDate()),
				pi.getState(),
				String.valueOf(pi.getRootProcessInstanceId()),
				String.valueOf(pi.getStartedBy()),
				String.valueOf(pi.getProcessDefinitionId()),
				String.valueOf(pi.getLastUpdate())
				);
		return caseObj;
	}

	
	/**
	 * Método que retorna los nombres de las variables y sus valores correspondientes las claves disponibles son: description, name, value, case_id, type
	 * @author José Francisco Luis Medina
	 * @param idCase identificador del caso
	 */
	@Override
	public  List<Map<String, String>> getCaseVarValues(String idCase){
		//this.sessionImpl.startConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
		List<Map<String,String>> listMap = new ArrayList<Map<String,String>>();
		try {
			//HttpResponse response = sessionImpl.executeGetRequest("/API/bpm/caseVariable?p=0&c=70&f=case_id%3d"+idCase);
			//listMap = this.jManager.arrayJSON2Map(response);
			Map<String,String> values=new HashMap<String,String>();			
			APISession  apiSession =   sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			List<DataInstance> datos=processAPI.getProcessDataInstances(Long.parseLong(idCase),0,100000);
			Iterator<DataInstance> datosIterator= datos.iterator();			
			//String value=null;
			while (datosIterator.hasNext()) {
				DataInstance dataInstance=datosIterator.next();	
				String value="";
				String className=dataInstance.getClassName().toString();
				if(className.equals("java.lang.Double")){
					value="0.0";
				}else if(className.equals("java.lang.Integer")
						||className.equals("java.lang.Long")){
					value="0";
				}
				//System.out.println(dataInstance.getName()+" "+value);
				if(dataInstance.getValue()!=null&&!dataInstance.getValue().toString().equals("")){
					value=dataInstance.getValue().toString();
				}
				values=new HashMap<String,String>();
				if(dataInstance.getName()!=null){
					values.put("case_id", idCase);
					values.put("name", dataInstance.getName().toString());
					values.put("value",value);
					values.put("type",dataInstance.getClassName().toString());
				    listMap.add(values);
				}
				
			
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return listMap;
	}
	
	
	/**
	 * Recibe un objeto del tipo ProcessInstance de Bonita y devuelve un objeto Case (bean del conector)
	 * TODO este método debe ser temporal ya que el ideal es que se manejen los objetos en sus tipos de datos primitivos
	 * @author José Francisco Luis Medina
	 * @param pi
	 * @return
	 */
	private Case beanCase2stringBeanCase(ProcessInstance pi){
		Case caseObj = new Case(
				String.valueOf(pi.getId()),
				String.valueOf(pi.getEndDate()),
				String.valueOf(pi.getStartDate()),
				pi.getState(),
				String.valueOf(pi.getRootProcessInstanceId()),
				String.valueOf(pi.getStartedBy()),
				String.valueOf(pi.getProcessDefinitionId()),
				String.valueOf(pi.getLastUpdate())
				);
		return caseObj;
	}
	
	/**
	 * Recibe un objeto del tipo ProcessInstance de Bonita y devuelve un objeto Case (bean del conector)
	 * TODO este método debe ser temporal ya que el ideal es que se manejen los objetos en sus tipos de datos primitivos
	 * @author José Francisco Luis Medina
	 * @param pi
	 * @return
	 */
	private Case beanArchivedCase2stringBeanCase(ArchivedProcessInstance api){
		Case caseObj = new Case(
				String.valueOf(api.getId()),
				String.valueOf(api.getEndDate()),
				String.valueOf(api.getStartDate()),
				api.getState(),
				String.valueOf(api.getRootProcessInstanceId()),
				String.valueOf(api.getStartedBy()),
				String.valueOf(api.getProcessDefinitionId()),
				String.valueOf(api.getLastUpdate())
				);
		return caseObj;
	}
	
	
	/**
	 * TODO  Este metodo se utiliza para obtener un listando del bean consulta gerencia de los procesos terminados, 
	 *  tiene como elementos de entrada un map que contiene todas o algunas variables transversales
	 *  @author Yamil Orozco
	 *  @param parameters: esta variable puede contener:
	 * <ol> 
	 * 		<li> La consulta por cedula es indivisual:</li>
	 *  	<li>clave cedula (string), valor numero de celuda (string)</li>
	 *  	<li> La consulta por id del proceso es individual:</li>
	 *  	<li>clave codigo (string), valor de la instancia de proceso (string)</li>
	 *      <li> por multiples valores:</li>
	 *      <li>clave tipoinstitucion (string), valor tipo de institucion(string)</li>
	 *      <li>clave institucionsolicitante(string), valor tipo de institucion(string)</li>
	 *      <li>clave nomina (string), valor nomina(string)</li>
	 *      <li>clave genero (string), valor genero(string)</li>
	 *      <li>clave motivo (string), valor motivo(string)</li>
	 *      <li>Nota: cuando consulte por fecha debe colocar la fecha inicial y final</li>
	 *      <li>clave fechasolicitudinicial (string), valor fecha de solicitud inicial(yyyy-MM-dd)(string)</li>
	 *      <li>clave fechasolicitudfinal (string), valor fecha de solicitud final (yyyy-MM-dd)(string)</li>
	 * </ol>
	 *  
	 *  
	 *  @return data : lista con el bean de ConsultaGenerica
	 *     <ol>
	 *     	 <li>id</li>
	 *       <li>cedula</li>
	 *       <li>proceso</li>
	 *       <li>nombre</li>
	 *       <li>apellido</li>
	 *       <li>fechaSolicitud</li>     
	 *       <li>fechaModificacion</li>
	 *     </ol> 
	 */

	public List<ConsultaGenerica> getCasesHistoryByParameters(Map<String,String> parameters){		
		
		JConnection jdbcConnetion=new JConnection();
		jdbcConnetion.openConnection();
		List<ConsultaGenerica> data=jdbcConnetion.executeQueryHistoryHead(parameters);
		jdbcConnetion.closeConnection();
		return data;		
	}
	
	/**
	 *   TODO Este metodo se utiliza para obtener un listando del bean consulta gerencia de los procesos en curso, 
	 *  tiene como elementos de entrada la cedula contribuyente
	 *  @author Yamil Orozco
	 *  @param cedula
	 *  @return data : lista con el bean de ConsultaGenerica	     
	 *     Valores que contiene el bean de ConsultaGenerica
	 *     <ol>
	 *     	 <li>id</li>
	 *       <li>cedula</li>
	 *       <li>proceso</li>
	 *       <li>nombre</li>
	 *       <li>apellido</li>
	 *       <li>fechaSolicitud</li>     
	 *       <li>fechaModificacion</li>
	 *     </ol> 
	 */

	public List<ConsultaGenerica> getCasesOnlineByCedula(String cedula){		
		
		JConnection jdbcConnetion=new JConnection();
		jdbcConnetion.openConnection();
		List<ConsultaGenerica> data=jdbcConnetion.executeQueryOnlineHead(cedula);
		jdbcConnetion.closeConnection();
		return data;		
	}
	
	
	/**
	 *  TODO Este metodo se utiliza para obtener un listando de las variables contenida en los procesos historicos, 
	 *  tiene como elemento de entrada el id del caso
	 *  @author Yamil Orozco
	 *  @param caseId
	 *  @return data List(map(String,String))     
	 *     <ol>
	 *     	 <li>name</li>
	 *       <li>value</li>
	 *       <li>type</li>
	 *     </ol> 
	 */

	public List<Map<String,String>> getVariablesHistoryByCaseId(String caseId){		
		
		JConnection jdbcConnetion=new JConnection();
		jdbcConnetion.openConnection();
		List<Map<String,String>> data=jdbcConnetion.executeQueryVariableHistory(caseId);
		jdbcConnetion.closeConnection();
		return data;		
	}
	
	
	/**
	 * Método que actualizar una variable es una instancia de caso
	 * @author José Francisco Luis Medina
	 * @param varName nombre de la variable a actualizar
	 * @param caseId identificador del caso
	 * @param varValue valor de la variable
	 */
	@Override
	public void updateCaseVariable(String varName, String caseId, Serializable varValue){
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			processAPI.updateProcessDataInstance(varName, Long.parseLong(caseId), varValue);
		} catch (Exception e) {
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
	}
			
	@SuppressWarnings({ "unchecked" })
	public  <T> T getVariable(String dataName,Long caseId){

		SessionImpl sessionImpl=new SessionImpl();
		APISession apiSession = null;
		ProcessAPI processAPI = null;
		DataInstance datos=null;
		T retorno=null;
		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			processAPI = sessionImpl.getProcessAPI(apiSession);
	    		datos=processAPI.getProcessDataInstance(dataName, caseId);

			sessionImpl.stopEngineConection(apiSession);
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DataNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if(datos.getValue()!=null){
			String value=null;
			String array[]=null;
			if(datos.getClassName().equals("java.util.Collection")){
				List<String> lista=new ArrayList<String>();
				value=datos.getValue().toString().replace("[", "").replace("]", "");
				array=value.split(",");
				//System.out.println(value);
				for(int i=0;i<array.length;i++){
					if(!array[i].isEmpty())
						lista.add(array[i]);
				}
				retorno=(T) lista;
			}

		}
		return retorno;
	}

	
	/*
	 * Este metodo devuelve los casos que estan en el historico
	 * 
	 */
	@Override
	public Case getCaseCompleted(String idCase) {
		// TODO Auto-generated method stub
		
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		Case caseObj = null;
		ArchivedProcessInstance archivedProcessInstance=null;
		try {			
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 100000);
			builder.filter(ArchivedProcessInstancesSearchDescriptor.SOURCE_OBJECT_ID, idCase);
			SearchResult<ArchivedProcessInstance> archivedProcessInstanceResults = null;
			try {
				archivedProcessInstanceResults = processAPI.searchArchivedProcessInstances(builder.done());
			} catch (SearchException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			};
			List<ArchivedProcessInstance> archivedProcessInstanceList=archivedProcessInstanceResults.getResult();
			archivedProcessInstance=archivedProcessInstanceList.get(0);
			
			caseObj = beanArchivedCase2stringBeanCase(archivedProcessInstance);	
			
			
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return caseObj;
		
		
	}



}
