package ve.gob.cnti.gestion.bonitaImpl;

import java.text.DateFormat;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.engine.bpm.comment.ArchivedComment;
import org.bonitasoft.engine.bpm.comment.ArchivedCommentsSearchDescriptor;
import org.bonitasoft.engine.bpm.comment.SearchCommentsDescriptor;
import org.bonitasoft.engine.exception.CreationException;
import org.bonitasoft.engine.exception.SearchException;
import org.bonitasoft.engine.platform.LoginException;
import org.bonitasoft.engine.search.Order;
import org.bonitasoft.engine.search.SearchOptionsBuilder;
import org.bonitasoft.engine.search.SearchResult;
import org.bonitasoft.engine.session.APISession;

import ve.gob.cnti.gestion.bonita.IComment;
import ve.gob.cnti.gestion.bonita.IUser;
import ve.gob.cnti.gestion.bonita.bean.Comment;
import ve.gob.cnti.gestion.utils.FacesHandler;

public class CommentImpl implements IComment {

	private FacesHandler facehandler = null;
	private List<Comment> comments = new ArrayList<Comment>();

	public CommentImpl() {

		new SessionImpl();
		this.facehandler = new FacesHandler();
	}

	/**
	 * Este método devuelve la lista de comentarios asociados a un caso particular
	 * @author José Francisco Luis Medina
	 * @param processID recibe el id del caso
	 * @return List<Comment> devuelve la lista de comentarios
	 */
	@Override
	public List<Comment> getComments(long processID) {

		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		comments = new ArrayList<Comment>();
		IUser user = new UserImpl();

		try {
			apiSession = sessionImpl.startEngineConection(facehandler
					.getUserSession().get("user"), facehandler.getUserSession()
					.get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 1000);
			builder.filter(SearchCommentsDescriptor.PROCESS_INSTANCE_ID, processID);
			builder.sort(SearchCommentsDescriptor.POSTDATE, Order.DESC);
			SearchResult<org.bonitasoft.engine.bpm.comment.Comment> commentsBonitaResults;
			commentsBonitaResults = processAPI.searchComments(builder.done());
			long id;
			
			for (org.bonitasoft.engine.bpm.comment.Comment commentBonita : commentsBonitaResults
					.getResult()) {
				
				//Valida si es un comentario del sistema y de ser así se lo asigna al usuario admin
				if (commentBonita.getUserId()==null) {
					 id = Long.parseLong(user.getIdByuserName("admin"));
				} else {
					id = commentBonita.getUserId();
					comments.add(new Comment(commentBonita.getContent(),
							commentBonita.getId(), convertTime(commentBonita.getPostDate()),
							commentBonita.getProcessInstanceId(), user.getUsernameById(id)));
				}
				
				
			}

		} catch (SearchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return comments;

	}
	
	/**
	 * Este método devuelve la lista de comentarios asociados a un caso finalizado
	 * @author José Francisco Luis Medina
	 * @param processID recibe el id del caso
	 * @return List<Comment> devuelve la lista de comentarios
	 */
	@Override
	public List<Comment> getCommentsCompleted(long processID) {

		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;
		comments = new ArrayList<Comment>();
		IUser user = new UserImpl();

		try {
			apiSession = sessionImpl.startEngineConection(facehandler
					.getUserSession().get("user"), facehandler.getUserSession()
					.get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 1000);
			builder.filter(ArchivedCommentsSearchDescriptor.PROCESS_INSTANCE_ID, processID);
			builder.sort(SearchCommentsDescriptor.POSTDATE, Order.DESC);
			SearchResult<org.bonitasoft.engine.bpm.comment.ArchivedComment> archivedCommentsBonitaResults;
			archivedCommentsBonitaResults = processAPI.searchArchivedComments(builder.done());
			long id;
			
			for (org.bonitasoft.engine.bpm.comment.ArchivedComment archivedCommentBonita : archivedCommentsBonitaResults
					.getResult()) {
				
				//Valida si es un comentario del sistema y de ser así se lo asigna al usuario admin
				if (archivedCommentBonita.getUserId()==null) {
					 id = Long.parseLong(user.getIdByuserName("admin"));
				} else {
					id = archivedCommentBonita.getUserId();
					comments.add(new Comment(archivedCommentBonita.getContent(),
							archivedCommentBonita.getId(), convertTime(archivedCommentBonita.getPostDate().getTime()),
							archivedCommentBonita.getProcessInstanceId(), user.getUsernameById(id)));
				}
				
				
			}

		} catch (SearchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return comments;

	}
	
	
	/**
	 * Agrega un comentario a un caso
	 * @author José Francisco Luis Medina
	 * @param processID recibe el id del caso
	 * @param comment comentario a incluir
	 */
	@Override
	public void addComment(long processID, String comment){
		
		SessionImpl sessionImpl = new SessionImpl();
		APISession apiSession = null;

		try {
			apiSession = sessionImpl.startEngineConection(facehandler
					.getUserSession().get("user"), facehandler.getUserSession()
					.get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			processAPI.addProcessComment(processID, comment);

		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	
	/**
	 * Convertir la fecha LONG que envía Bonita a formato legible para la UI
	 * @author José Francisco Luis Medina
	 * @param time
	 * @return
	 */
	private String convertTime(long time){
	    Date date = new Date(time);
	    DateFormat format = new SimpleDateFormat("dd/MM/yyyy  HH:mm:ss");
	    format.setTimeZone(TimeZone.getTimeZone("GMT-0430")); 
	    return format.format(date);
	}

}
