package ve.gob.cnti.gestion.bonitaImpl;

import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONException;

import ve.gob.cnti.gestion.bonita.ICase;
import ve.gob.cnti.gestion.bonita.IController;
import ve.gob.cnti.gestion.bonita.IProcess;
import ve.gob.cnti.gestion.bonita.ITask;
import ve.gob.cnti.gestion.bonita.IUser;
import ve.gob.cnti.gestion.bonita.bean.Task;
import ve.gob.cnti.gestion.formfile.FolderManager;
import ve.gob.cnti.gestion.utils.FacesHandler;


public class ControllerImpl implements IController{
	
	public IProcess process = null;
	public ICase caseObj = null;
	public ITask task = null;
	public FacesHandler facehandler = null;
	public IUser user = null;


	public ControllerImpl(){
		process = new ProcessImpl();
		caseObj = new CaseImpl();
		task = new TaskImpl();
		facehandler = new FacesHandler();
		user = new UserImpl();

	}
	/**
	 * INICIA UN CASO, ASIGNA LA PRIMERA TAREA A QUIEN LO INICIO Y RETORNA UN MAP CON EL NOMBRE DE LA PRIMERA TAREA PENDIENTE (nameTask) Y EL ID DEL CASO CREADO (idCase)
	 * @param idProc
	 * @return Map<String, String> keys=> idCase, nameTask
	 * @throws UnsupportedEncodingException
	 * @throws IllegalStateException
	 * @throws JSONException
	 * @throws IOException
	 * @throws InterruptedException 
	 */
	public Map<String, String> initCase(String idProc, String cedula) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException, InterruptedException{
		System.out.println("->Inciando proceso de nuevo caso");
		Map<String,String> mp=new HashMap<String, String>();
		//Long idCase = (Long) process.startProcess(idProc, cedula).get("idCase");
		Map<String,Object> caseMap = process.startProcess(idProc, cedula);
		Long idCase = (Long) caseMap.get("idCase");
		System.out.println("->Terminando StartProcess");
		System.out.println("======================================");
		System.out.println("idCase=> "+idCase);
		Task taskObj = task.getTaskbyCaseId(String.valueOf(idCase));
		System.out.println("TASK => "+ taskObj.toString());
		ve.gob.cnti.gestion.bonita.bean.ProcessDefinition procDef = process.getProcess(idProc);
		mp.put("nameTask", taskObj.getName());
		mp.put("idTask", taskObj.getId());
		mp.put("userHasCase", caseMap.get("userHasCase").toString());
		System.out.println("NAME PROC => "+procDef.getDisplayName());
		mp.put("nameProc", procDef.getDisplayName());
		System.out.println("======================================");
		if(mp.get("userHasCase").contentEquals("false")){
			task.assignTask(caseObj.getCase(String.valueOf(idCase)).getStarted_by(), taskObj.getId());
			FolderManager folderManager=new FolderManager(String.valueOf(idCase));
			folderManager.createDir();
		}
		//guardo el id del caso de una session y se crea el directorio temporal en carga de datos
		
		//FacesHandler faceHandler=new FacesHandler();
		//faceHandler.setParam("idCase", String.valueOf(idCase));
		
		System.out.println("paso por el inicio del proceso");
		
		//
		return mp;
	}
	
//	@Override
//	public void completeTask(String idTask, List<Map<String, Object>> results){
//		JsonManager jManager = new JsonManager();
//		JSONObject jObject = jManager.listMap2payload(results);
//		String payload = jObject.toString();
//		task.updateTaskVariables(idTask, payload);
//		task.executeTask(idTask);
//		
//	}
	
	@Override
	public void completeTask(String idTask, List<Map<String, Object>> results){
		Map<String, Serializable> variables = new HashMap<String, Serializable>(); 
		
		for (Map<String, Object> map : results) {
		    for (Map.Entry<String, Object> entry : map.entrySet()) {
		        variables.put(entry.getKey(), (Serializable) entry.getValue());
		    }
		}
		
		task.updateTaskVariables(idTask, variables);
		task.executeTask(idTask);
		
	}
	
	/**
	 * Este método recibe el id de la tarea a realizar, la asigna al usuario en sesión y devuelve el nombre de la tarea pendiente para que sea renderizado el formulario
	 * @author Jose Francisco Luis Medina
	 * @param idTask id de la tarea a realizar
	 * @throws IOException 
	 * @throws JSONException 
	 * @throws IllegalStateException 
	 * @throws UnsupportedEncodingException 
	 */
	@Override
	public String assignAndDoTask(String idTask) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		task.assignTask(facehandler.getUserSession().get("user"), idTask);
		return task.getTask(idTask).getName();
	}
	
	/**
	 * Este metodo obtiene la informacion del contenido de las variables de un caso a través del id del caso indicado
	 * @author José Francisco Luis Medina
	 * @param idCase
	 * @return
	 * 
	 */
	@Override
	public List<Map<String, String>> getVariablesValues(String idCase){
		return caseObj.getCaseVarValues(idCase);
	}
	
	/**
	 * Devuelve un listado de tareas disponibles para un usuario filtradas por proceso
	 * @author José Francisco Luis Medina
	 * @param {@link String} idUser {@link idProcessDefinition}
	 * @return {@link List}<{@link Task}> availableTask
	 */
	@Override
	public List<Task> gesTasksAvailableForUser(String idUser){
		ITask task = new TaskImpl();
		List<Task> availableTasks = task.gesTasksAvailableForUser(idUser);
		return availableTasks;
	}
}
