package ve.gob.cnti.gestion.bonitaImpl;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.bonitasoft.engine.api.IdentityAPI;
import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.engine.bpm.process.ProcessInstance;
import org.bonitasoft.engine.bpm.process.ProcessInstanceSearchDescriptor;
import org.bonitasoft.engine.exception.AlreadyExistsException;
import org.bonitasoft.engine.exception.CreationException;
import org.bonitasoft.engine.exception.SearchException;
import org.bonitasoft.engine.exception.UpdateException;
import org.bonitasoft.engine.identity.Group;
import org.bonitasoft.engine.identity.GroupSearchDescriptor;
import org.bonitasoft.engine.identity.Role;
import org.bonitasoft.engine.identity.RoleSearchDescriptor;
import org.bonitasoft.engine.identity.UserMembershipCriterion;
import org.bonitasoft.engine.identity.UserNotFoundException;
import org.bonitasoft.engine.identity.UserUpdater;
import org.bonitasoft.engine.platform.LoginException;
import org.bonitasoft.engine.search.SearchOptionsBuilder;
import org.bonitasoft.engine.search.SearchResult;
import org.bonitasoft.engine.session.APISession;
import org.bonitasoft.engine.session.InvalidSessionException;
import org.json.JSONException;

import ve.gob.cnti.gestion.bonita.IActorMember;
import ve.gob.cnti.gestion.bonita.IProcess;
import ve.gob.cnti.gestion.bonita.IUser;
import ve.gob.cnti.gestion.bonita.bean.Actor;
import ve.gob.cnti.gestion.bonita.bean.ActorMember;
import ve.gob.cnti.gestion.bonita.bean.Membership;
import ve.gob.cnti.gestion.bonita.bean.ProcessDefinition;
import ve.gob.cnti.gestion.bonita.bean.User;
import ve.gob.cnti.gestion.utils.FacesHandler;
import ve.gob.cnti.gestion.utils.PropertiesReader;

public class UserImpl implements IUser {
	
	PropertiesReader properties = new PropertiesReader();		
    public String ADMIN_USERNAME = properties.bonitaProperties("ADMIN_USERNAME");
    public String ADMIN_PASSWORD = properties.bonitaProperties("ADMIN_PASSWORD");
    public String NOMBRE_ROL_CIUDADANO = properties.bonitaProperties("NOMBRE_ROL_CIUDADANO");
    public String NOMBRE_GRUPO_CIUDADANO = properties.bonitaProperties("NOMBRE_GRUPO_CIUDADANO");

	private SessionImpl sessionImpl = null;
	private FacesHandler facehandler = null;
	private IProcess processDef = null;
	private IActorMember actorMemberDef = null;

	public UserImpl() {

		sessionImpl = new SessionImpl();
		facehandler = new FacesHandler();
		processDef = new ProcessImpl();
		actorMemberDef = new ActorMemberImpl();		

	}
	
	/**
	 * Devuelve el id del usuario cuando se le entrega el username 
	 * @author José Francisco Luis Medina
	 * @param {@link String} userName
	 * @return {@link String} idUser
	 */
	@Override
	public String getIdByuserName(String userName){
		
		APISession apiSession = null;
		org.bonitasoft.engine.identity.User u = null;
		IdentityAPI identityAPI = null;
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			identityAPI = SessionImpl.getIdentityAPI(apiSession);
			u = identityAPI.getUserByUserName(userName);
		} catch (LoginException e) {
			e.printStackTrace();
		} catch (UserNotFoundException e) {
//			e.printStackTrace();
			createUser(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			try {
				u = identityAPI.getUserByUserName(userName);
			} catch (UserNotFoundException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
	
		sessionImpl.stopEngineConection(apiSession);
		return Long.toString(u.getId());
	}
	
	
	/**
	 * Devuelve el usuario del cuando se le entrega el id 
	 * @author José Francisco Luis Medina
	 * @param {@link String} idUser
	 * @return {@link String} username
	 */
	@Override
	public String getUsernameById(long userId){
		
		APISession apiSession = null;
		org.bonitasoft.engine.identity.User u = null;
		IdentityAPI identityAPI = null;
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			identityAPI = SessionImpl.getIdentityAPI(apiSession);
			u = identityAPI.getUser(userId);
		} catch (LoginException e) {
			e.printStackTrace();
		} catch (UserNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return u.getUserName();
	}
	
	

	/**
	 * Este metodo devuelve las membresias de un usuario
	 * 
	 * @author José Francisco Luis Medina
	 * @param idUser
	 * @return
	 */
	private List<Membership> getMembership(String idUser) {
	List<Membership> listMemberships = new ArrayList<Membership>();
	APISession apiSession = null;
	
	try {
		apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
		IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
		identityAPI = SessionImpl.getIdentityAPI(apiSession);
		List<org.bonitasoft.engine.identity.UserMembership> engineMemberships =  identityAPI.getUserMemberships(Long.parseLong(idUser), 0, 100, UserMembershipCriterion.ASSIGNED_DATE_ASC);
		Iterator<org.bonitasoft.engine.identity.UserMembership> ite = engineMemberships.iterator();
		while (ite.hasNext()) {
			listMemberships.add(beanMembership2stringMembership(ite.next()));
		}
		
	} catch (LoginException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
		sessionImpl.stopEngineConection(apiSession);
		return listMemberships;
	}

	/**
	 * Devuelve un listado de los actores asociados al usuario indicado para "todos los procesos"
	 * @author José Francisco Luis Medina
	 * @param {@link String} idUser
	 * @return {@link List}<{@link Actor}>
	 */
	@Override
	public List<Actor> getActorByUser(String idUser) {
		List<Actor> listActor = new ArrayList<Actor>();
		try {
			for (ProcessDefinition proc : processDef.listProcess()) {
				listActor.addAll(this.getActorByUserOnAProcess(idUser, proc.getId()));
			}
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		};
		return listActor;
	}

	/**
	 * Este metodo devuelve los actores a los que esta asociado un usuario para el proceso indicado
	 * @author José Francisco Luis Medina
	 * @param idUser
	 * @return
	 */
	@Override
	public List<Actor> getActorByUserOnAProcess(String idUser,
			String idProcessDefinition) {
		ArrayList<Actor> listActorsUser = new ArrayList<Actor>();
		List<Actor> listActorsGeneral = processDef
				.getActors(idProcessDefinition);
		List<Membership> listMemberships = this.getMembership(idUser);
		Iterator<Actor> ite = listActorsGeneral.iterator();
		while (ite.hasNext()) {
			Actor actor = ite.next();
			List<ActorMember> listActorMember = actorMemberDef
					.listActorMembers(actor.getId());
			Iterator<ActorMember> ite2 = listActorMember.iterator();
			while (ite2.hasNext()) {
				ActorMember actorMember = ite2.next();
				Iterator<Membership> ite3 = listMemberships.iterator();
				while (ite3.hasNext()) {
					Membership membership = ite3.next();
					if (membership.getRole_id()
							.equals(actorMember.getRole_id())
							&& membership.getGroup_id().equals(
									actorMember.getGroup_id())) {
						listActorsUser.add(actor);
					}
				}
			}
		}
		return listActorsUser;
	}
	
	
	@Override
	/**
	 * Únicamente crea un usuario, no sin asignarle perfil ni membresía
	 * @author José Francisco Luis Medina
	 * @param {@link String}username {@link String}password
	 * @return {@link String} idUser
	 * TODO VERIFICAR SI LOS USUARIOS DEBEN SER CREADOS EXCLUSIVAMENTE POR UN USUARIO ADMIN DE BONITA
	 */
	public String createUser(String username, String password){
		org.bonitasoft.engine.identity.User u = null;
		APISession apiSession = null;
		String idUser = null;
		
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			u = identityAPI.createUser(username, password);
			idUser = Long.toString(u.getId());
			addMembershipToUser(idUser);
			
		} catch (AlreadyExistsException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			updateUserPassword(username, password);
		} catch (CreationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return idUser;
	}
	
	/**
	 * Anade la membresia de ciudadano al usuario indicado
	 * @author José Francisco Luis Medina
	 * @param {@link String}
	 * @return void
	 */
	private void addMembershipToUser(String userId){
		APISession apiSession = null;
	
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			identityAPI.addUserMembership(Long.parseLong(userId), getCiudadanoGroupId(), getCiudadanoRoleId());
		} catch (AlreadyExistsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
	}
	
	
	/**
	 * Devuelve el ID asignado al grupo Ciudadano
	 * @author José Francisco Luis Medina
	 * @param {@link String}
	 * @return {@link String}
	 */
	private Long getCiudadanoGroupId(){
		APISession apiSession = null;
		Long idGroup = null; 
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 100);
			builder.filter(GroupSearchDescriptor.NAME, NOMBRE_GRUPO_CIUDADANO);
			SearchResult<Group> group = identityAPI.searchGroups(builder.done());
			for (Group gp : group.getResult()) {
				//System.out.println("Nombre: "+gp.getName()+" => ID: "+gp.getId());
				idGroup = gp.getId();
			}
			
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SearchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return idGroup;
	}
	
	
	/**
	 * Devuelve el ID asignado al rol Ciudadano
	 * @author José Francisco Luis Medina
	 * @param {@link String}
	 * @return {@link String}
	 */
	private Long getCiudadanoRoleId(){
		APISession apiSession = null;
		Long idRole = null; 
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			SearchOptionsBuilder builder = new SearchOptionsBuilder(0, 100);
			builder.filter(RoleSearchDescriptor.NAME, NOMBRE_ROL_CIUDADANO);
			SearchResult<Role> role = identityAPI.searchRoles(builder.done());
			for (Role rl : role.getResult()) {
				//System.out.println("Nombre: "+rl.getName()+" => ID: "+rl.getId());
				idRole = rl.getId();
			}
			
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SearchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		return idRole;
	}
	
	
	/**
	 * Actualiza el password del usuario indicado con el password enviado por los parametros
	 * @author José Francisco Luis Medina
	 * @param username
	 * @param password
	 * TODO el manejo del id del usuario no debe ser por String sino por su dato primitivo Long
	 */
	public void updateUserPassword(String username, String password){
		SessionImpl sessionImpl = new SessionImpl();
		UserUpdater uu = new UserUpdater();
		APISession apiSession = null;
		try {
		
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			Long idUser = Long.parseLong(getIdByuserName(username));
			identityAPI.updateUser(idUser, uu.setPassword(password));
			
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UserNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UpdateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		
	}
	
	@Override
	/**
	 * Devuelve el listado de usuarios candidatos a ejecutar la tarea indicada
	 * @author José Francisco Luis Medina
	 * @param idTask
	 * @return
	 */
	public List<User> getUsersAvailablesForTask(String idTask) {
		
		APISession apiSession = null;
		List<org.bonitasoft.engine.identity.User> listUsers = new ArrayList<org.bonitasoft.engine.identity.User>();
		List<User> availableUsers = new ArrayList<User>();
		try {
			apiSession = sessionImpl.startEngineConection(facehandler.getUserSession().get("user"), facehandler.getUserSession().get("pass"));
			ProcessAPI processAPI = sessionImpl.getProcessAPI(apiSession);
			listUsers = processAPI.getPossibleUsersOfPendingHumanTask(Long.parseLong(idTask), 0, 100);
			 
			for (org.bonitasoft.engine.identity.User user : listUsers) {
				availableUsers.add(bonitaUser2stringBeanUser(user));
			}
			
		} catch (LoginException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		sessionImpl.stopEngineConection(apiSession);
		
		return availableUsers;
	}
	
	/**
	 * TODO este método debe desaparecer ya que en un futuro se debe trabajar con los tipos de datos primitivos del Engine
	 * Devuelve un objeto User del Bean String Definido al recibir un objeto User del Engine de Bonita
	 * @author José Francisco Luis Medina
	 * @param user
	 * @return
	 */
	private User bonitaUser2stringBeanUser (org.bonitasoft.engine.identity.User user){
		User userString = new User(
				String.valueOf(user.getLastConnection()),
				String.valueOf(user.isEnabled()),
				String.valueOf(user.getIconPath()),
				String.valueOf(user.getManagerUserId()),
				String.valueOf(user.getLastName()),
				String.valueOf(user.getFirstName()),
				String.valueOf(user.getPassword()),
				String.valueOf(user.getId()),
				String.valueOf(user.getCreationDate()),
				String.valueOf(user.getCreatedBy()),
				String.valueOf(user.getTitle()),
				String.valueOf(user.getJobTitle()),
				String.valueOf(user.getUserName()),
				String.valueOf(user.getLastUpdate())
				); 
		return userString;
	} 
	
	
	/**
	 * Devuelve un objeto Membership del Bean String Definido al recibir un objeto UserMembership del Engine de Bonita
	 * @author José Francisco Luis Medina
	 * @param user
	 * @return
	 */
	private Membership beanMembership2stringMembership(org.bonitasoft.engine.identity.UserMembership uem) {
		Membership um = new Membership(
				uem.getAssignedDate().toString(),
				Long.toString(uem.getRoleId()),
				Long.toString(uem.getAssignedBy()),
				Long.toString(uem.getGroupId()),
				Long.toString(uem.getUserId())
				);
		return um;
	}
	
	
}
