package ve.gob.cnti.gestion.controller;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import org.json.JSONException;

import ve.gob.cnti.gestion.bonita.ICase;
import ve.gob.cnti.gestion.bonita.IComment;
import ve.gob.cnti.gestion.bonita.IProcess;
import ve.gob.cnti.gestion.bonita.IUser;
import ve.gob.cnti.gestion.bonita.bean.Case;
import ve.gob.cnti.gestion.bonita.bean.Comment;
import ve.gob.cnti.gestion.bonitaImpl.CaseImpl;
import ve.gob.cnti.gestion.bonitaImpl.CommentImpl;
import ve.gob.cnti.gestion.bonitaImpl.ProcessImpl;
import ve.gob.cnti.gestion.bonitaImpl.UserImpl;
import ve.gob.cnti.gestion.controller.beans.CasesHelper;
import ve.gob.cnti.gestion.utils.FacesHandler;
import ve.gob.cnti.gestion.utils.Utils;

@ManagedBean(name = "casesController", eager = true)
@SessionScoped
public class CasesController {
	
	private FacesHandler facesHandler = new FacesHandler();
	private ICase iCases = new CaseImpl();
	private IProcess iProcess = new ProcessImpl();
	private Case caseObj = new Case();
	private Utils utils = new Utils();
	private List<Case> listCases = new ArrayList<Case>();
	private String processName;
	private IUser iUser = new UserImpl();
	private IComment iComment = new CommentImpl();
	private String comment = "";
	private List<Comment> comments;
		

	/**Obtiene la Lista de TrÃ¡mites abiertos por el usuario
	 * @author Joaquin Pereira
	 * @return Lista de TrÃ¡mites abiertos por el usuario
	 */
	public List<CasesHelper> getListCasesOpened() throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		List<CasesHelper> chList = new ArrayList<CasesHelper>();
		
		List<Case> cases = iCases.listCases(iUser.getIdByuserName(facesHandler.getUserSession().get("user")));
		for(Case caso : cases){
			CasesHelper ch = new CasesHelper();
			ch.setId(caso.getId());
			ch.setProcessDefinitionId(caso.getProcessDefinitionId());
			ch.setProcessDisplayName(new ProcessController().getProcessDisplayName(caso.getProcessDefinitionId()));
			ch.setProcessName(new ProcessController().getProcessName(caso.getProcessDefinitionId()));
			ch.setActivityPending(ch.obtenerTareaPendiente(caso.getId()));
			ch.setStartDate(utils.getFormattedDate("dd-MM-yyyy HH:mm:ss", caso.getStart()));			
			chList.add(ch);
		}
		
		return chList;
	}
	
	/**Obtiene el TrÃ¡mite abierto por el usuario
	 * @author Joaquin Pereira
	 * @return TrÃ¡mite abierto por el usuario
	 */
	public String getCaseOpened(String id,String view) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		//facesHandler.setUserSession("ciudadano", "123");
		caseObj =  iCases.getCase(id);		
		return "/views/"+view+"/objCase.xhtml?faces-redirect=true";
	}
	
	
	public Case getCase(String id) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		caseObj =  iCases.getCase(id);
		return caseObj;
	}
	
	public Case getCaseHistory(String id) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		caseObj =  iCases.getArchivedCase(id);
		return caseObj;
	}
	
	
	public String getCaseCompleted(String id,String view) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		caseObj =  iCases.getCaseCompleted(id);			
		return "/views/"+view+"/objCaseCompleted.xhtml?faces-redirect=true";
	}
	
	
	public Case getCaseCompleted(String id){
		caseObj =  iCases.getCaseCompleted(id);
		return caseObj;
		
	}
	
	/**Obtiene el resúmen de un caso indicado
	 * @author José Francisco Luis Medina
	 */
	public String getSummaryCase(String idCase) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		caseObj =  iCases.getCase(idCase);
		comments = getListComments(idCase);
		return "/views/interfazportalfuncionario/case/summary.xhtml";
	}
	
	public String addComment(String idCase){
		iComment.addComment(Long.parseLong(idCase), comment);
		comment="";
		String redirect = "/views/interfazportal"+FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get("role")+"/objCase.xhtml?faces-redirect=true"; 
		return redirect;
	}
	
	public Boolean addCommentOnActuation(){
		System.out.println("Caso: "+facesHandler.getParam("idCase"));		
		iComment.addComment(Long.parseLong(facesHandler.getParam("idCase")), facesHandler.getParam("comment"));
		comment="";
		return true;
	}
	
	/**Obtiene los comentarios actuales
	 * 
	 * @param idcaso
	 * @return lista de comentario
	 */
	
	
	
	public List<Comment> getListComments(String idCaso){
		List<Comment> comments = iComment.getComments(Long.parseLong(idCaso));
		return comments;
	}
	
	/**Obtiene los comentarios de los procesos finalizados
	 * 
	 * @param idcaso
	 * @return lista de comentarios
	 */
	
	
	public List<Comment> getListCommentsCompleted(String idCaso){
		List<Comment> comments = iComment.getCommentsCompleted(Long.parseLong(idCaso));
		return comments;
	}
	
	
	/**Obtiene El nombre del trÃ¡mite abierto por el ciudadano
	 * 
	 * @param idProcess
	 * @return El nombre del trÃ¡mite abierto por el ciudadano
	 */
	public String getTramiteName(String idProcess) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{		
		//facesHandler.setUserSession("ciudadano", "123");
		return iProcess.getProcess(idProcess).getDescription();
	}
	
	/**Obtiene la fecha de creaciÃ³n del trÃ¡mite 
	 * 
	 * @param idProcess
	 * @return la fecha de creaciÃ³n del trÃ¡mite (proceso)
	 * @throws IOException 
	 * @throws JSONException 
	 * @throws IllegalStateException 
	 * @throws UnsupportedEncodingException 
	 */
	public String getStartCase(String idCase) throws UnsupportedEncodingException, IllegalStateException, JSONException, IOException{
		caseObj = iCases.getCase(idCase);
		return utils.getFormattedDate("dd-MM-yyyy -- HH:mm:ss",caseObj.getStart());
	}
	
	public String getApplicantName(String idCase){
		String nombre="",apellido="";
		List<Map<String,String>> lista = iCases.getCaseVarValues(idCase);
		Iterator<Map<String,String>> mapas = lista.iterator();
		while(mapas.hasNext()){
			Map<String,String> map = mapas.next();
			Iterator<String> it = map.keySet().iterator();
			while (it.hasNext()) {
				String key = it.next();
				if(key.contentEquals("name") && map.get(key).contentEquals("b_nombre"))
					nombre = map.get(it.next());
				if(key.contentEquals("name") && map.get(key).contentEquals("c_apellido"))
					apellido = map.get(it.next());
				if(nombre.length()>0 && apellido.length()>0)
					break;
			}
			if(nombre.length()>0 && apellido.length()>0)
				break;
		}
		return nombre + " " +apellido;
	}
	
	
	public Case getCaseObj() {
		return caseObj;
	}
	public void setCaseObj(Case caseObj) {
		this.caseObj = caseObj;
	}
	public List<Case> getListCases() {
		return listCases;
	}
	public void setListCases(List<Case> listCases) {
		this.listCases = listCases;
	}	

	public String getProcessName() {
		return processName;
	}

	public void setProcessName(String processName) {
		this.processName = processName;
	}
	
	
	public Utils getUtils() {
		return utils;
	}

	public void setUtils(Utils utils) {
		this.utils = utils;
	}
	
	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public List<Comment> getComments() {
		return comments;
	}

	public void setComments(List<Comment> comments) {
		this.comments = comments;
	}
	
	
	
}
