package ve.gob.cnti.gestion.controller.beanFiles;

import java.util.List;
import java.util.Map;

import org.primefaces.component.fileupload.FileUpload;
import org.primefaces.model.UploadedFile;

import ve.gob.cnti.gestion.utils.FacesHandler;
import ve.gob.cnti.gestion.utils.Utils;

/**
 * Clase para servir con funcionalidades a los manejos de archivos.
 * 
 * @author Joaquin Pereira
 * 
 */
public class BeanFileServices {

	private Utils util = new Utils();
	private BeanFile fileBean;

	public BeanFile obtenerBeanFile(Map<String, Integer> archivosProcesados,
			FileUpload fileUpload, List<BeanFile> archivos) {
		if (archivosProcesados.containsKey(fileUpload.getId())) {
			return archivos.get(archivosProcesados.get(fileUpload.getId()));
		} else {
			return creacionDeBeanDeArchivo(fileUpload);
		}
	}

	public boolean archivoValido(BeanFile archivo, UploadedFile file,
			List<BeanFile> archivos) {
		if (util.filePDFValidateFirmado(file, "")) {
			return false;
		}

		if (archivo.isMultiple()
				&& archivo.getFileNames().size() == archivo
						.getTotalDeArchivos()) {
			util.setMessageToView(
					"Error: se alzanzo el limite de documentos permitidos para "
							+ archivo.getNombreOficial(), 1);
			return false;
		}

		if (existeArchivoEnLaLista( file.getFileName(),	archivos))
			return false;

		return true;
	}

	private boolean existeArchivoEnLaLista(String fileName,
			List<BeanFile> archivos) {
		for (BeanFile beanFiles : archivos) {
			if (beanFiles.getFileNames().contains(fileName)) {
				util.setMessageToView("Error el archivo: " + fileName
						+ " ya fue subido", 1);
				return true;
			}
		}
		return false;
	}

	private BeanFile creacionDeBeanDeArchivo(FileUpload fileUpload) {
		fileBean = new BeanFile();
		fileBean.setObligatorio(fileUpload.isRequired());
		fileBean.setPesoDelArchivo(fileUpload.getSizeLimit());
		fileBean.setVarName(fileUpload.getId());
		fileBean.setNombreOficial(fileUpload.getLabel());
		fileBean.setMultiple(fileUpload.isMultiple());
		if (fileBean.isMultiple())
			fileBean.setTotalDeArchivos(Integer.parseInt((String) fileUpload
					.getAttributes().get("limiteDeArchivos")));
		fileBean.setTab(ObtenerTab(fileUpload.getId()));
		return fileBean;
	}

	private String ObtenerTab(String var) {
		FacesHandler facesHandler = new FacesHandler();
		String temp = (String) facesHandler.findComponent("files")
				.getAttributes().get("tabsNames");
		String tabs[] = temp.split(",");		
		temp = (String) facesHandler.findComponent("files").getAttributes()
				.get("tabs");
		int nTabs = Integer.parseInt(temp);
		for (int i = 0; i < nTabs; i++) {
			temp = (String) facesHandler.findComponent("files").getAttributes()
					.get(tabs[i]);
			String archivos[] = temp.split(",");
			for(String archivo: archivos){
				if(archivo.contentEquals(var)){
					return tabs[i];
				}
			}
		}
		return "";
	}

}
