package ve.gob.cnti.gestion.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import ve.gob.cnti.gestion.bonita.bean.ConsultaGenerica;
import ve.gob.cnti.gestion.bonitaImpl.CaseImpl;
import ve.gob.cnti.gestion.utils.PropertiesReader;
import ve.gob.cnti.gestion.utils.Utils;

@ManagedBean(name = "historyConsult", eager = true)
@SessionScoped
public class historyConsult {

	private String id;
	private String cedula;
	private String proceso;
	private String nomina;
	private String genero;
	private String motivo;
	private String institucionsolicitante;
	private String tipoinstitucion;
	private Date fechaSolicitud;
	private Date fechaModificacion;

	private boolean doRequest;
	private String parametros;

	private List<ConsultaGenerica> listHistoryCases = new ArrayList<ConsultaGenerica>();
	private CaseImpl caseImpl = new CaseImpl();
	private Map<String, String> mapValues = new HashMap<String, String>();
	private Utils utils = new Utils();

	public String listCases() {
		System.out.println(listHistoryCases.size());
		System.out.println(this.cedula);
		listHistoryCases.clear();
		if (this.cedula != null) {
			
			listHistoryCases = caseImpl
					.getCasesOnlineByCedula(this.getCedula());
			if (listHistoryCases.size() == 0)
				utils.setMessageToView("La CI: " + this.cedula
						+ " no tiene solicitudes activas.", 4);
			this.cedula = "";
		}
		System.out.println(listHistoryCases.size());
		System.out.println(this.cedula);
		return "/views/layout/consultas/buscarPorCI.xhtml";
	}

	public String buscarCasosPorParametros() {
		listHistoryCases.clear();
		if (validarExistenciaDeAlgunPerametro()) {
			this.parametros="";
			mapValues.clear();
			
			ingresarClaveValorAlMapa("codigo", this.id, "ID");
			ingresarClaveValorAlMapa("cedula", this.cedula, "CI");
			ingresarClaveValorAlMapa("genero", this.genero, "Género");
			ingresarClaveValorAlMapa("tipoinstitucion", this.tipoinstitucion,
					"Tipo de Organismo");
			ingresarClaveValorAlMapa("institucionsolicitante",
					this.institucionsolicitante, "Organismo");
			ingresarClaveValorAlMapa("motivo", this.motivo, "Motivo");
			// ingresarClaveValorAlMapa("proceso", this.proceso,"");
			ingresarClaveValorAlMapa("nomina", this.nomina, "Nómina");
			ingresarFechasAlMapa(fechaSolicitud, fechaModificacion);
			if (this.doRequest)
				listHistoryCases = caseImpl
						.getCasesHistoryByParameters(mapValues);
			if (listHistoryCases.size() == 0)
				utils.setMessageToView(
						new PropertiesReader().getI18nProperty("historyEmpty")
								+ " -- " + this.parametros, 4);
			clearVars();
		}

		return "/views/layout/consultas/buscarPorParametros.xhtml";
	}

	private boolean validarExistenciaDeAlgunPerametro() {
		boolean a = true, b = true, c = true;
		try {
			if (this.fechaModificacion.toString() != null) {
				a = true;
			}
		} catch (Exception e) {
			a = false;
		}
		try {
			if (this.fechaSolicitud.toString() != null) {
				b = true;
			}
		} catch (Exception e) {
			b = false;
		}
		if (this.id == null && this.cedula == null && this.proceso == null
				&& this.nomina == null && this.genero == null
				&& this.motivo == null && this.institucionsolicitante == null
				&& this.tipoinstitucion == null) {
			c = false;
		} else {
			c = true;
		}
		if (!a && !b && !c) {
			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error!",
							"Debe elegir un tipo de consulta"));
			return false;
		}
		return true;
	}

	private void ingresarClaveValorAlMapa(String clave, String valor,
			String label) {
		if (valor != null) {
			if (!valor.isEmpty()) {
				this.doRequest = true;
				mapValues.put(clave, valor);
				parametros += "("+label + ": " + valor + ") -- ";
			}
		}
	}

	private void ingresarFechasAlMapa(Date fechaIncio, Date fechaFin) {
		if (fechaIncio != null && fechaFin != null) {
			// TODO ingresar valores al mapa3
			if (fechaIncio.compareTo(fechaFin) > 0) {
				FacesContext
						.getCurrentInstance()
						.addMessage(
								null,
								new FacesMessage(
										FacesMessage.SEVERITY_ERROR,
										"Error!",
										"La fecha de solicitud debe ser menor o igual a la fecha de la última modificación"));
			} else {
				SimpleDateFormat formatter = new SimpleDateFormat("yyyy-M-dd",
						Locale.US);
				mapValues.put("fecha_inicio",
						formatter.format(fechaIncio));
				mapValues
						.put("fecha_fin", formatter.format(fechaFin));
				this.doRequest = true;
				parametros += "(Fecha de Solicitud: "
						+ formatter.format(fechaIncio) + ") -- ";
				parametros += "(Fecha de Modificación: "
						+ formatter.format(fechaIncio) + ")";
			}
		} else if (fechaIncio != null && fechaFin == null) {
			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error!",
							"Debe indicar ambas fechas."));
		} else if (fechaIncio == null && fechaFin != null) {
			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "Error!",
							"Debe indicar ambas fechas."));
		}
	}

	private void clearVars() {
		this.id = "";
		this.cedula = "";
		this.proceso = "";
		this.motivo = "";
		this.tipoinstitucion = "";
		this.genero = "";
		this.nomina = "";
		this.institucionsolicitante = "";
		this.fechaModificacion = null;
		this.fechaSolicitud = null;
		this.doRequest = false;
		this.parametros = "";
	}

	public void clean(){
		listHistoryCases.clear();
	}
	
	public List<ConsultaGenerica> getListHistoryCases() {
		return listHistoryCases;
	}

	public void setListHistoryCases(List<ConsultaGenerica> listHistoryCases) {
		this.listHistoryCases = listHistoryCases;
	}

	public String getCedula() {
		if (cedula == null)
			return "";
		else
			return cedula;
	}

	public void setCedula(String cedula) {
		this.cedula = cedula;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getProceso() {
		return proceso;
	}

	public void setProceso(String proceso) {
		this.proceso = proceso;
	}

	public String getNomina() {
		return nomina;
	}

	public void setNomina(String nomina) {
		this.nomina = nomina;
	}

	public String getGenero() {
		return genero;
	}

	public void setGenero(String genero) {
		this.genero = genero;
	}

	public String getMotivo() {
		return motivo;
	}

	public void setMotivo(String motivo) {
		this.motivo = motivo;
	}

	public String getInstitucionsolicitante() {
		return institucionsolicitante;
	}

	public void setInstitucionsolicitante(String institucionsolicitante) {
		this.institucionsolicitante = institucionsolicitante;
	}

	public String getTipoinstitucion() {
		return tipoinstitucion;
	}

	public void setTipoinstitucion(String tipoinstitucion) {
		this.tipoinstitucion = tipoinstitucion;
	}

	public Date getFechaSolicitud() {
		return fechaSolicitud;
	}

	public void setFechaSolicitud(Date fechaSolicitud) {
		this.fechaSolicitud = fechaSolicitud;
	}

	public Date getFechaModificacion() {
		return fechaModificacion;
	}

	public void setFechaModificacion(Date fechaModificacion) {
		this.fechaModificacion = fechaModificacion;
	}

}
