package ve.gob.cnti.gestion.io;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.rmi.RemoteException;
import java.util.Map;

import ve.gob.cnti.gestion.bonita.IIO;
import ve.gob.cnti.gestion.bonita.bean.DatosBasicosNatural;
import ve.gob.cnti.gestion.io.saime.BuscaTripletaPorSujetoService;
import ve.gob.cnti.gestion.io.saime.BuscaTripletaPorSujetoServiceProxy;
import ve.gob.cnti.gestion.io.saime.LinkedDataException;
import ve.gob.cnti.gestion.io.saime.Tripleta;
import ve.gob.cnti.gestion.rest.ciudadano.ImplRestCiudadano;
import ve.gob.cnti.gestion.utils.PropertiesReader;
import ve.gob.cnti.gestion.utils.Utils;
import ve.gob.cnti.gestion.utils.annotations.Fieldkey;
public class IOImpl implements IIO {

	PropertiesReader properties = new PropertiesReader();
	/**
	 * Método que devuelve una instancia de DatosBasicosNatural llena con los datos consultados desde SAIME a través de la plataforma
	 * de IO, en dicho bean no se llenan datos como pais, estado, municipio y parroquia, los cuales son consultados desde el sistema
	 * de registro de usuarios
	 *
	 * @author José Francisco Luis Medina & Joaquin Pereira
	 * @param cedula
	 * @return
	 */
	public DatosBasicosNatural getDataCiudadano(String cedula) {

		DatosBasicosNatural db = new DatosBasicosNatural();
		//String sujeto = SAIME_HOST + SAIME_SERVICE + nac + "-" + cedula;
		ImplRestCiudadano restCiudadano=new ImplRestCiudadano();
		Map<String, String> map=restCiudadano.getDataCiudadano(cedula);
		Field[] campos = db.getClass().getDeclaredFields();
		Utils utils=new Utils();
		try {
			for (Field f : campos) {
				f.setAccessible(true);

				if (!"serialVersionUID".equals(f.getName())) {
				if ("primerNombre".contentEquals(f.getName()))
					f.set(db,map.get("nombre"));
				else if ("primerApellido".contentEquals(f.getName()))
					f.set(db,map.get("apellido"));
				else if ("cedula".contentEquals(f.getName()))
					f.set(db,(map.get("cedula").toUpperCase().replace("V", "").replace("E", "")));
				else if("f_nacimiento".contentEquals(f.getName())){
					f.set(db, utils.getSympleFormattedDate("dd/MM/yyyy", "yyyy-MM-dd",map.get(f.getName())));
				}
				else if (map.get(f.getName())!=null)
					f.set(db, map.get(f.getName()));
				else
					f.set(db,"");
				}
			}
		} catch (IllegalArgumentException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		} catch (IllegalAccessException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		}

		return db;
	}

	/**
	 * Método que devuelve una instancia de DatosBasicosNatural llena con los datos consultados con todos los sistemas
	 * que interoperan con la plataforma de tramites
	 *
	 * @author José Francisco Luis Medina
	 * @param nac
	 * @param cedula
	 * @return
	 */
	@Override
	public DatosBasicosNatural getDataNatural(String nac, String cedula) {
		return getDataSaime(nac, cedula);
	}

	public String SAIME_HOST = properties.IOProperties("SAIME_HOST");

	public String SAIME_SERVICE = properties.IOProperties("SAIME_SERVICE");

	/**
	 * Método que devuelve una instancia de DatosBasicosNatural llena con los datos consultados desde SAIME a través de la plataforma
	 * de IO, en dicho bean no se llenan datos como pais, estado, municipio y parroquia, los cuales son consultados desde el sistema
	 * de registro de usuarios
	 *
	 * @author José Francisco Luis Medina & Joaquin Pereira
	 * @param nac
	 * @param cedula
	 * @return
	 */
	private DatosBasicosNatural getDataSaime(String nac, String cedula) {

		BuscaTripletaPorSujetoService buscar = new BuscaTripletaPorSujetoServiceProxy();
		Tripleta[] tri = null;
		DatosBasicosNatural db = new DatosBasicosNatural();

		try {
			String sujeto = SAIME_HOST + SAIME_SERVICE + nac + "-" + cedula;
			tri = buscar.buscaTripletaPorSujeto(sujeto);
			String nacionalidad = "";

			for (Tripleta tripleta : tri) {
				System.out.println(tripleta.getPropiedad() + " " +
						tripleta.getValor());
				Field[] campos = db.getClass().getDeclaredFields();

				for (Field f : campos) {
					f.setAccessible(true);
					Annotation[] anotaciones = f.getAnnotations();

					for (Annotation anotacion : anotaciones) {
						if (anotacion instanceof Fieldkey) {
							Fieldkey fk = (Fieldkey)anotacion;

							if (fk.key().contentEquals(tripleta.getPropiedad())) {
								f.set(db, tripleta.getValor());

							}

							if ("NUMCEDULA".contentEquals(tripleta
									.getPropiedad()) &&
								fk.key().contentEquals("NUMCEDULA")) {
								f.set(db,
										tripleta.getValor().substring(2,
												tripleta.getValor().length()));
								nacionalidad = String.valueOf(tripleta.getValor().charAt(0));
							}

							if ("NACIONALIDAD".contentEquals(tripleta
									.getPropiedad()) &&
								fk.key().contentEquals("NACIONALIDAD")) {
								//Se coloca así debido a que los arreglos vienen desde IO en ordenes variables//

								if ("VEN".contentEquals(tripleta.getValor())) {
									f.set(db,"V");
								}else {
									f.set(db,"E");
								}
								//////////////////////////////////////////////////////////////////////////////////

							}

							if ("COD_EDO_CIVIL".contentEquals(tripleta
									.getPropiedad())
									&& fk.key().contentEquals("COD_EDO_CIVIL")) {
								f.set(db,
										properties.IOProperties(tripleta.getValor()));
							}

						}
					}
				}
			}

		} catch (LinkedDataException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		} catch (RemoteException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		} catch (IllegalArgumentException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		} catch (IllegalAccessException e) {

			// TODO Auto-generated catch block

			e.printStackTrace();
		}

		System.out.println(db.toString());

		return db;
	}

}