package ve.gob.cnti.gestion.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.dbcp2.BasicDataSource;
import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.engine.bpm.data.ArchivedDataInstance;
import org.bonitasoft.engine.bpm.flownode.ArchivedHumanTaskInstance;
import org.bonitasoft.engine.bpm.flownode.HumanTaskInstance;
import org.bonitasoft.engine.bpm.process.ArchivedProcessInstance;
import org.bonitasoft.engine.bpm.process.ProcessDefinition;
import org.bonitasoft.engine.bpm.process.ProcessInstance;
import org.bonitasoft.engine.bpm.process.ProcessInstanceSearchDescriptor;
import org.bonitasoft.engine.platform.LoginException;
import org.bonitasoft.engine.search.SearchOptionsBuilder;
import org.bonitasoft.engine.search.SearchResult;
import org.bonitasoft.engine.session.APISession;

import ve.gob.cnti.gestion.bonita.bean.ConsultaGenerica;
import ve.gob.cnti.gestion.bonitaImpl.SessionImpl;
import ve.gob.cnti.gestion.jms.ConnectionJMS;
import ve.gob.cnti.gestion.jms.IConnectionJMS;
import ve.gob.cnti.gestion.utils.PropertiesReader;
import ve.gob.cnti.gestion.utils.Utils;

/**
 * Esta clase se encarga de extraer informacion de la tabla arch_process_instance_lastupdate, process_instance y data_instance
 * para implementar esta clase utilice la clase CaseImpl.java
 * @author Yamil Orozco
 */

public class JConnection {
	
	Connection cnn=null;
    PreparedStatement ps=null;
    ResultSet rs=null;
    BasicDataSource basicDataSource=null;   
    PropertiesReader properties = new PropertiesReader();		
    public String DATABASE_URL = properties.getJDBCProperty("DATABASE_URL");
    public String DATABASE_USER = properties.getJDBCProperty("DATABASE_USER");
    public String DATABASE_PASSWORD = properties.getJDBCProperty("DATABASE_PASSWORD");
    public Integer MAX_CONNECTION = Integer.valueOf(properties.getJDBCProperty("MAX_CONNECTION"));
    public Integer INITIAL_CONNECTION = Integer.valueOf(properties.getJDBCProperty("INITIAL_CONNECTION"));
    public Integer MAX_IDLE = Integer.valueOf(properties.getJDBCProperty("MAX_IDLE"));
    public Integer MIN_IDLE = Integer.valueOf(properties.getJDBCProperty("MIN_IDLE"));
    public Boolean REMOVE_ABANDONED = Boolean.valueOf(properties.getJDBCProperty("REMOVE_ABANDONED"));
    public Integer REMOVE_ABANDONED_TIME = Integer.valueOf(properties.getJDBCProperty("REMOVE_ABANDONED_TIME"));
    
	
    public String ADMIN_USERNAME = properties.bonitaProperties("ADMIN_USERNAME");
    public String ADMIN_PASSWORD = properties.bonitaProperties("ADMIN_PASSWORD");
    SessionImpl sessionImpl = new SessionImpl();
	APISession apiSession=null;
	ProcessAPI processAPI=null;
	ProcessInstance processInstance=null;	
	List<ArchivedDataInstance> listArchivedDataInstance=null;	
    Boolean next=null;
    
    public JConnection(){
    	
    }
    
    /**
     * abre la conexion con la base de datos, es llamado del constructor
     *
     */    
    
	public void openConnection(){
		
    	basicDataSource=new BasicDataSource();
		basicDataSource.setDriverClassName("org.postgresql.Driver");
		basicDataSource.setUrl(DATABASE_URL);
		basicDataSource.setUsername(DATABASE_USER);
		basicDataSource.setPassword(DATABASE_PASSWORD);
		basicDataSource.setMaxTotal(MAX_CONNECTION);
		basicDataSource.setInitialSize(INITIAL_CONNECTION);
		basicDataSource.setValidationQuery("select 1");
		basicDataSource.setRemoveAbandonedOnBorrow(REMOVE_ABANDONED);
		basicDataSource.setRemoveAbandonedTimeout(REMOVE_ABANDONED_TIME);
		basicDataSource.setMaxIdle(MAX_IDLE);
		basicDataSource.setMinIdle(MIN_IDLE);	
		
		
	
	}
	
    /**
     * extrae informacion de los historicos de los procesos
     * @param parameters
     * @return data
     */ 
	public List<ConsultaGenerica> executeQueryHistoryHead(Map<String,String> parameters){		
		
		//este metodo extrae de el historico o actual de los procesos por cedula o numero de tramite
		
		List<ConsultaGenerica> data=new ArrayList<ConsultaGenerica>();
		ConsultaGenerica consultaGenerica=null;
		try {				
		
			String query=null;				
			Utils utils=new Utils();
			String patternsOut="dd-MM-yyyy";
			String patternsIn="yyyy-MM-dd";
			query="select * from proceso_instancias_hist where true ";	

			
			if(parameters.get("codigo")!=null){
				query+=" and id_instancia=?";									
			}
			
			if(parameters.get("cedula")!=null){
				query+=" and trim(cedula)=?";									
			}			
				
			if(parameters.get("nombre")!=null){
				query+=" and trim(nombre) like ? ";								
			}								
			if(parameters.get("apellido")!=null){
				query+=" and trim(apellido) like ? ";																
			}								
			if(parameters.get("sexo")!=null){
				query+=" and trim(sexo)=? ";
			}								
			if(parameters.get("fecha_nac")!=null){
				query+=" and fecha_nac>=to_date(?,'YYYY-MM-DD') ";
			}								
			if(parameters.get("pais_org")!=null){
				query+=" and trim(pais_orig)=? ";
			}
			
			if(parameters.get("estado_org")!=null){
				query+=" and trim(estado_orig)=? ";
			}
			
			if(parameters.get("ciudad_org")!=null){
				query+=" and trim(ciudad_orig)=? ";
			}
			if(parameters.get("tramite")!=null){
				query+=" and trim(tramite)=? ";
			}
			
			if(parameters.get("fecha_inicio")!=null){						
				query+=" and fecha_inicio>=to_date(?,'YYYY-MM-DD') ";
			}
			
			if(parameters.get("fecha_fin")!=null){						
				query+=" and fecha_fin<= to_date(?,'YYYY-MM-DD') ";
			}
			
			//System.out.println(query);
			cnn =basicDataSource.getConnection();
			ps = cnn.prepareStatement(query);
			
			int i=0;
			
			if(parameters.get("codigo")!=null){						
				ps.setLong(++i,Long.valueOf(parameters.get("codigo")));								
			}
			
			if(parameters.get("cedula")!=null){
				ps.setString(++i,parameters.get("cedula"));												
			}
					

			
			if(parameters.get("nombre")!=null){						
				ps.setString(++i,"%"+parameters.get("nombre")+"%");								
			}
					
			if(parameters.get("apellido")!=null){
				ps.setString(++i,"%"+parameters.get("apellido")+"%");																
			}								
			if(parameters.get("sexo")!=null){				
				ps.setString(++i,parameters.get("sexo"));								
			}								
			if(parameters.get("fecha_nac")!=null){				
				ps.setString(++i,parameters.get("fecha_nac"));									
			}								
			if(parameters.get("pais_org")!=null){
				ps.setString(++i,parameters.get("pais_org"));																
			}								
			if(parameters.get("estado_org")!=null){
				ps.setString(++i,parameters.get("estado_org"));									
			}
			if(parameters.get("ciudad_org")!=null){
				ps.setString(++i,parameters.get("ciudad_org"));									
			}
			if(parameters.get("tramite")!=null){
				ps.setString(++i,parameters.get("tramite"));									
			}								 
			if(parameters.get("fecha_inicio")!=null){
				ps.setString(++i,parameters.get("fecha_inicio"));						
									
			}	
			if(parameters.get("fecha_fin")!=null){
				ps.setString(++i,parameters.get("fecha_fin"));														
									
			}
			
			
			rs = ps.executeQuery();
			String fecha_inicio;
			String fecha_fin;		
			
			
			while(rs.next()){

				fecha_inicio=utils.getSympleFormattedDate(patternsOut,patternsIn,rs.getDate("fecha_inicio").toString());
				fecha_fin=utils.getSympleFormattedDate(patternsOut,patternsIn,rs.getDate("fecha_fin").toString());				
				consultaGenerica=new ConsultaGenerica(String.valueOf(rs.getLong("id_instancia")),rs.getString("cedula"),
						rs.getString("tramite"), rs.getString("nombre"),rs.getString("descripcion_proceso"),rs.getString("apellido"), fecha_inicio, fecha_fin);							
				data.add(consultaGenerica);
								
								
			}
				
	  } catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		
	  }finally{
		  try {
				if(rs!=null)rs.close();
				if(ps!=null) ps.close();
				if(cnn!=null) cnn.close();
		   } catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
		   }
	  }
		
      return data;
		
	}	
    /**
     * extrae informacion de los procesos en curso
     * @param cedula
     * @return data
     */ 
	public List<ConsultaGenerica> executeQueryOnlineHead(String cedula){
		
		List<ConsultaGenerica> data=new ArrayList<ConsultaGenerica>();
		ConsultaGenerica consultaGenerica=null;	
		SearchOptionsBuilder builder=null;
		try {
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			processAPI = sessionImpl.getProcessAPI(apiSession);
			
			
		} catch (LoginException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try{
			String query=null;				
			Utils utils=new Utils();
			String patterns="dd-MM-yyyy";
			query="select d1.id as codigo,d2.name as name,concat(cast(d2.intvalue as varchar),"
				    +"cast(d2.shorttextvalue as varchar),cast(d2.longvalue as varchar),"
					+"cast(d2.booleanvalue as varchar),cast(d2.doublevalue as varchar),"
					+"cast(d2.floatvalue as varchar),cast(d2.clobvalue as varchar),"
					+"cast(d2.blobvalue as varchar)) as cedula, d2.classname as tipo,"
					+"d1.lastupdate as lastupdate,d1.name as nombreproceso,"
					+"d1.description as descripcion,concat(cast(d3.intvalue as varchar),cast(d3.shorttextvalue as " 
					+"varchar),cast(d3.longvalue as varchar),cast(d3.booleanvalue as " 
					+"varchar),cast(d3.doublevalue as varchar),cast(d3.floatvalue as "
					+"varchar),cast(d3.clobvalue as varchar),cast(d2.blobvalue as varchar)"
					+") as nombre,concat(cast(d4.intvalue as varchar),"
					+"cast(d4.shorttextvalue as varchar),cast(d4.longvalue as varchar),"
					+"cast(d4.booleanvalue as varchar),cast(d4.doublevalue as varchar),"
					+"cast(d4.floatvalue as varchar),cast(d4.clobvalue as varchar),"
					+"cast(d4.blobvalue as varchar)) as " 
					+"apellido from process_instance d1,data_instance d2,"
					+"data_instance d3,data_instance d4 where d1.id=d2.containerid and " 
					+"d1.id=d3.containerid and d1.id=d4.containerid and (d2.name='cedula' "
					+"and d2.intvalue=?) and d3.name='nombre'and d4.name='apellido' order by d1.id";			
					cnn =basicDataSource.getConnection();
					ps = cnn.prepareStatement(query);
					ps.setInt(1,Integer.valueOf(cedula));
					rs = ps.executeQuery();
					String fechaSolicitud;
					String fechaActualizacion;					
					while(rs.next()){
						builder = new SearchOptionsBuilder(0, 1);
						builder.filter(ProcessInstanceSearchDescriptor.ID, rs.getLong("codigo"));						
						SearchResult<ProcessInstance> processInstanceResults=processAPI.searchProcessInstances(builder.done());;
						List<ProcessInstance> processInstanceList=processInstanceResults.getResult();
						processInstance=processInstanceList.get(0);						
						fechaSolicitud=utils.getSympleFormattedDate("dd-MM-yyyy", "EEE MMM dd HH:mm:ss z yyyy", processInstance.getStartDate().toString());
						fechaActualizacion=utils.getFormattedDate(patterns,(new Date(rs.getLong("lastupdate")).toString()));
						consultaGenerica=new ConsultaGenerica(String.valueOf(rs.getLong("codigo")),rs.getString("cedula")
								,rs.getString("nombreproceso"),rs.getString("nombre"),rs.getString("apellido"),
								fechaSolicitud,fechaActualizacion);							
					    data.add(consultaGenerica);		
						
					}
		
		}catch(Exception e){
			e.printStackTrace();
		}finally{		
			 try {
				if(rs!=null)rs.close();
				if(ps!=null) ps.close();
				if(cnn!=null) cnn.close();
			
			 } catch (SQLException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
			 }
		}
		
		return data;
	}
	
	
    /**
     * se extrae informacion de las variables de los procesos finalizados
     * @param caseId
     * @return listaData
     */ 
	public List<Map<String,String>> executeQueryVariableHistory(String caseId){
		 
		 List<Map<String,String>> listaData=new ArrayList<Map<String,String>>();
		 Utils utils=new Utils();		 
		 ConsultaGenerica consultaGenerica=null;
		 
		try {	
			Map<String,String> parameters=new HashMap<String,String>();
			JConnection jdbcConnection=new JConnection();
			jdbcConnection.openConnection();
			parameters.put("codigo", caseId);
			List<ConsultaGenerica> consultaGeneralLista=jdbcConnection.executeQueryHistoryHead(parameters);
			jdbcConnection.closeConnection();;
			consultaGenerica=consultaGeneralLista.get(0);
			apiSession = sessionImpl.startEngineConection(ADMIN_USERNAME, ADMIN_PASSWORD);
			processAPI = sessionImpl.getProcessAPI(apiSession);
		} catch (LoginException e1) {
		// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}	
		
		List<ArchivedDataInstance>	archivesData=processAPI.getArchivedProcessDataInstances(Long.valueOf(caseId), 0, 100000);
		ArchivedDataInstance data=null;
		Iterator<ArchivedDataInstance> archivesDataIterator=archivesData.iterator();
		Map<String,String> register=null;
		String value=null;
		while(archivesDataIterator.hasNext()){
			data=(ArchivedDataInstance)archivesDataIterator.next();
			register=new HashMap<String,String>();
			if(data.getValue()!=null&&data.getClassName().equals("java.util.Date"))
				value=utils.getSympleFormattedDate("EEE MMM dd HH:mm:ss z yyyy", "EEE MMM dd kk:mm:ss z yyy",data.getValue().toString()
						.replace("<?xml version=\"1.0\" ?><date>", "").
						replace("</date>", "").trim()).toString();				    
			else if(data.getValue()!=null)
				value= String.valueOf(data.getValue());	
			else
				value="";
				register.put("id",caseId);				
				register.put("processname",consultaGenerica.getProceso());
				register.put("name",data.getName());
				register.put("value",value);
				register.put("type",data.getClassName());
				listaData.add(register);	
				
			} 
		
		//sessionImpl.stopConection();
	
		return listaData;
	}

    /**
     * cierra la conexion a la base de datos
     */ 
	public void closeConnection(){
		
		try {
			/*ps.close();
			cnn.close();*/
			if (basicDataSource!=null)
				basicDataSource.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
    
	

}
