package ve.gob.cnti.gestion.user;

import ve.gob.cnti.gestion.bonitaImpl.SessionImpl;
import ve.gob.cnti.gestion.rest.ciudadano.IRestCiudadano;
import ve.gob.cnti.gestion.rest.ciudadano.ImplRestCiudadano;
import ve.gob.cnti.gestion.user.User;

import java.io.IOException;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;

import ve.gob.cnti.gestion.utils.LdapManager;
import ve.gob.cnti.gestion.utils.PropertiesReader;

import org.bonitasoft.engine.api.IdentityAPI;
import org.bonitasoft.engine.identity.ContactData;
import org.bonitasoft.engine.identity.UserNotFoundException;
import org.bonitasoft.engine.platform.LoginException;
import org.bonitasoft.engine.session.APISession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@ManagedBean
@SessionScoped
public class UserManager {

	private static final Logger LOGGER = LoggerFactory.getLogger(UserManager.class);

	public static final String HOME_FUNCIONARIO_REDIRECT = "/views/interfazportalfuncionario/view.xhtml?faces-redirect=true";
	public static final String HOME_CIUDADANO_REDIRECT = "/views/interfazportalciudadano/view.xhtml?faces-redirect=true";
	public static final String LOGOUT_PAGE_REDIRECT_CIUDADANO = "/login/ciudadano.xhtml?faces-redirect=true";
	public static final String LOGOUT_PAGE_REDIRECT_FUNCIONARIO = "/login/funcionario.xhtml?faces-redirect=true";
	public static final String ROLE_CIUDADANO = "ciudadano";
	public static final String ROLE_FUNCIONARIO = "funcionario";
	public static final String HOME_CENTRAL_LOGIN = "/login/home.xhtml?faces-redirect=true";

	private String userId;
	private String userPassword;
	private String role;
	private User currentUser;
	
	PropertiesReader properties = new PropertiesReader();		
    public String ADMIN_USERNAME = properties.bonitaProperties("ADMIN_USERNAME");
    public String ADMIN_PASSWORD = properties.bonitaProperties("ADMIN_PASSWORD");

	public String loginCiudadano() {
		userId = Character.toUpperCase(userId.charAt(0)) + userId.substring(1);
		currentUser = buscarCiudadano(userId, userPassword);
		if (currentUser != null) {
			LOGGER.info("inicio de sesión exitóso para '{}'", userId);
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("userId", currentUser.getUserId());
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("pass", ve.gob.cnti.gestion.utils.Utils.getMD5(userPassword));
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("role", currentUser.getRole());
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("email", currentUser.getEmail());
			return HOME_CIUDADANO_REDIRECT;
		} else {
			LOGGER.info("inicio de sesión fallido para '{}'", userId);
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_WARN,
					"Inicio de sesión fallido", "Usuario o Password inválido."));

			return null;
		}
	}
	
	public String loginFuncionario() {

		currentUser = buscarFuncionario(userId, userPassword);
		if (currentUser != null) {
			LOGGER.info("inicio de sesión exitóso para '{}'", userId);
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("userId", currentUser.getUserId());
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("pass", userPassword);//TODO revisar manejo de password de creación de usuarios funcionarios
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("role", currentUser.getRole());
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("email", currentUser.getEmail());
			return HOME_FUNCIONARIO_REDIRECT;
		} else {
			LOGGER.info("inicio de sesión fallido para '{}'", userId);
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_WARN,
					"Inicio de sesión fallido", "Usuario o Password inválido."));

			return null;
		}
	}

	public String logout() {
		
		String identifier = userId;
		
		// invalidate the session
		LOGGER.debug("cerrando la sesión para '{}'", identifier);
		FacesContext.getCurrentInstance().getExternalContext().invalidateSession();
		LOGGER.info("cierre de sesión exitóso para '{}'", identifier);
		
		if(insertFuncionarioComponentsOnView()){
			
			return LOGOUT_PAGE_REDIRECT_FUNCIONARIO;
			
		}else if(insertCiudadanoComponentsOnView()){
			
			return LOGOUT_PAGE_REDIRECT_CIUDADANO;
		}
		
		return null;
	}

	public boolean isLoggedIn() {
		return currentUser != null;
	}

	public String isLoggedInForwardHome() {

		if (isLoggedIn() && ROLE_FUNCIONARIO
				.equals(currentUser.getRole())) {
			return HOME_FUNCIONARIO_REDIRECT;
		} else if (isLoggedIn() && ROLE_CIUDADANO
				.equals(currentUser.getRole())) {
			return HOME_CIUDADANO_REDIRECT;
		}
		return null;
	}

	/**
	 * Consulta usuarios del LDAP de registro ciudadano para permitir el acceso a la bandeja ciudadano
	 * @author José Francisco Luis Medina
	 * @param userId
	 * @param password
	 * @return
	 */
	private User buscarCiudadano(String userId, String password) {
		User result = null;
		try {
			if (LdapManager.authenticateJndi(userId, password)) {
				
				//Búsqueda de los datos de registro ciudadano
				IRestCiudadano datosc = new ImplRestCiudadano();
				Map<String, String>mapc = datosc.getDataCiudadano(userId.toString());

				result = new User(userId, mapc.get("nombre"), mapc.get("apellido"), ROLE_CIUDADANO, mapc.get("email"));
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
	
	
	/**
	 * Consulta usuarios de bonita para permitir el acceso a la bandeja funcionario
	 * @author José Francisco Luis Medina 
	 * @param userId
	 * @param password
	 * @return user
	 */
	public User buscarFuncionario(String userId, String password){
		User result = null;
		try {
			SessionImpl session = new SessionImpl();
			APISession apiSession = session.startEngineConection(userId, password, ROLE_FUNCIONARIO);
			IdentityAPI identityAPI = SessionImpl.getIdentityAPI(apiSession);
			org.bonitasoft.engine.identity.User user = identityAPI.getUserByUserName(userId);
			ContactData contactdata = identityAPI.getUserContactData(user.getId(), true);
			result = new User(userId, user.getFirstName(), user.getLastName(), ROLE_FUNCIONARIO, contactdata.getEmail());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	// Es funcionario?
	public void isFuncionario(ComponentSystemEvent event) {

		//LOGGER.debug("Verificando el Rol del usuario");

		if (!"funcionario".equals(currentUser.getRole())) {
			try {
				String identifier = userId;
				LOGGER.debug("Usuario con permisos insuficientes cerrando la sesión para '{}'", identifier);
				ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
				ec.responseSendError(401, "Permisos Insuficientes"); // TODO
																		// redireccionar
																		// message
																		// a la
																		// vista
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}
	
	// Es ciudadano?
		public void isCiudadano(ComponentSystemEvent event) {

			//LOGGER.debug("Verificando el Rol del usuario");

			if (!"ciudadano".equals(currentUser.getRole())) {
				try {
					String identifier = userId;
					LOGGER.debug("Usuario con permisos insuficientes cerrando la sesión para '{}'", identifier);
					ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
					ec.responseSendError(401, "Permisos Insuficientes"); // TODO
																			// redireccionar
																			// message
																			// a la
																			// vista
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		}
		
		// Válida el rol para ingresar los componentes de ciudadano en la vista
				public Boolean insertCiudadanoComponentsOnView() {
					//LOGGER.debug("Verificando el Rol del usuario");
					if ("ciudadano".equals(currentUser.getRole())) {
						return true;
					}else{
						return false;
					}
				}
		// Válida el rol para ingresar los componentes de funcionario en la vista
				public Boolean insertFuncionarioComponentsOnView() {
					//LOGGER.debug("Verificando el Rol del usuario");
					if ("funcionario".equals(currentUser.getRole())) {
						return true;
					}else{
						return false;
					}
				}
	
	public String getUserId() {
		return userId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}

	public String getUserPassword() {
		return userPassword;
	}

	public void setUserPassword(String userPassword) {
		this.userPassword = userPassword;
	}

	public User getCurrentUser() {
		return currentUser;
	}

	public String getRole() {
		return role;
	}

	public void setRole(String role) {
		this.role = role;
	}

}
