package ve.gob.cnti.gestion.utils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.bean.ManagedBean;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.visit.VisitCallback;
import javax.faces.component.visit.VisitContext;
import javax.faces.component.visit.VisitResult;
import javax.faces.context.FacesContext;
import com.sun.faces.component.visit.FullVisitContext;

@ManagedBean(name = "FacesHandler", eager = true)
public class FacesHandler {
	
	/** Obtiene un atributo enviado en una petición
	 * @author Joaquin Pereira	
	 * @param key Clave atributo a buscar en la petición
	 * @return valor del atributo buscado en la petición
	 */
	public String getParam(String key){
		FacesContext fc = FacesContext.getCurrentInstance();		
		Map<String,String> params = fc.getExternalContext().getRequestParameterMap();
		return params.get(key);
	}	
	
	/** Obtiene un atributo de session
	 * @author Joaquin Pereira	
	 * @param key Clave atributo a buscar en la petición
	 * @return valor del atributo buscado en la petición
	 */
	public String getSession(String key){
		FacesContext fc = FacesContext.getCurrentInstance();		
		Map<String,Object> params = fc.getExternalContext().getSessionMap();
		return (String)params.get(key);
	}
	
	
	/** Crea una variable de session
	 * @author Joaquin Pereira	
	 * @param key Clave atributo a generar
	 * @return null
	 */
	public void setParam(String key, String value){
		
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(key, value);

	}	
	
	
	
	
	/**Obtiene los datos de la sessión de JSF (Estos datos son obtenidos manualmente, falta recuperarlos de la session de liferay)
	 * @author Joaquin Pereira
	 * @return un mapa de string clave-valor (keys ==> {user,pass,email})
	 * @throws SystemException 
	 * @throws PortalException 
	 */
	public Map<String,String> getUserSession(){		
		Map<String,String> map = new HashMap<String, String>();
			map.put("user", FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get("userId").toString());
			map.put("pass", FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get("pass").toString());
			map.put("email", FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get("email").toString());	
		return map;
		
	}
	
	/**Guarda en el ambito de la sesion el usuario y clave (Estos datos son seteados manualmente, falta recuperarlos de la session de liferay)
	 * @author Joaquin Pereira
	 * @param user nombre del usuario
	 * @param pass clave del usuario
	 */
	public void setUserSession(String user,String pass){
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("user", user);
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("pass", pass);
	}
	
	/**Obtiene de la petición un elemento de la vista mediante su id
	 * @author Joaquin Pereira
	 * @param id del componente java faces a conseguir
	 * @return {@link UIComponent} Componente de java faces buscado
	 */
	public UIComponent findComponent(final String id) {

	    FacesContext context = FacesContext.getCurrentInstance(); 
	    UIViewRoot root = context.getViewRoot();
	    final UIComponent[] found = new UIComponent[1];

	    root.visitTree(new FullVisitContext(context), new VisitCallback() {     
	        @Override
	        public VisitResult visit(VisitContext context, UIComponent component) {
	            if(component.getId().equals(id)){
	                found[0] = component;
	                return VisitResult.COMPLETE;
	            }
	            return VisitResult.ACCEPT;              
	        }
	    });

	    return found[0];

	}
	
	/**Elimina del ámbito de la sesión de java server faces el bean
	 * 
	 * @param beanELName nombre del bean definido en el ManagedBean de la clase
	 * 
	 * @author Joaquin Pereira
	 */
	public void removeBeanFromSessionScoped(String beanELName){
		Map<String, Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
		
		try {
			sessionMap.remove(beanELName); 
			System.out.println("Objeto: "+beanELName+" removido de la sesion jsf");
		} catch (Exception e) {
			// TODO: handle exception
		}
	}
	
	/**Envia a la pagina de error que le notifica al usuario que su navegador no tiene habilitado el java
	 * Script
	 * 
	 * @author Joaquin Pereira
	 * 
	 * @throws IOException
	 */
	public void noJsScript() throws IOException{		
		FacesContext fc = FacesContext.getCurrentInstance();
	    ConfigurableNavigationHandler nav = (ConfigurableNavigationHandler)fc.getApplication().getNavigationHandler();
	    nav.performNavigation("/views/errorNoJS.xhtml");
	}
	
}
