package ve.gob.cnti.gestion.utils;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.dbcp2.BasicDataSource;

import ve.gob.cnti.gestion.bonita.bean.ConsultaGenerica;

/**
 * Esta clase se encarga de extraer informacion de la tabla arch_process_instance_lastupdate, process_instance y data_instance
 * para implementar esta clase utilice la clase CaseImpl.java
 * @author Yamil Orozco
 */

public class JConnection {
	
	Connection cnn=null;
    PreparedStatement ps=null;
    ResultSet rs=null;
    BasicDataSource basicDataSource=null;   
    PropertiesReader properties = new PropertiesReader();		
    public String DATABASE_URL = properties.getJDBCProperty("DATABASE_URL");
    public String DATABASE_USER = properties.getJDBCProperty("DATABASE_USER");
    public String DATABASE_PASSWORD = properties.getJDBCProperty("DATABASE_PASSWORD");
    public Integer MAX_CONNECTION = Integer.valueOf(properties.getJDBCProperty("MAX_CONNECTION"));
    public Integer INITIAL_CONNECTION = Integer.valueOf(properties.getJDBCProperty("INITIAL_CONNECTION"));

    Boolean next=null;
    
    public JConnection(){
    	this.openConnection();
    }
    
    /**
     * abre la conexion con la base de datos, es llamado del constructor
     *
     */    
    
	public void openConnection(){
		
		basicDataSource= new BasicDataSource();		
		basicDataSource.setDriverClassName("org.postgresql.Driver");
		basicDataSource.setUrl(DATABASE_URL);
		basicDataSource.setUsername(DATABASE_USER);
		basicDataSource.setPassword(DATABASE_PASSWORD);
		basicDataSource.setMaxTotal(MAX_CONNECTION);
		basicDataSource.setInitialSize(INITIAL_CONNECTION);
		basicDataSource.setValidationQuery("select 1");
		
		
		try {
			cnn =basicDataSource.getConnection();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
    /**
     * extrae informacion de los historicos de los procesos
     * @param parameters
     * @return data
     */ 
	public List<ConsultaGenerica> executeQueryHistoryHead(Map<String,String> parameters){		
		
		//este metodo extrae de el historico o actual de los procesos por cedula o numero de tramite
		
		List<ConsultaGenerica> data=new ArrayList<ConsultaGenerica>();
		ConsultaGenerica consultaGenerica=null;
		try {
			String query=null;				
			Utils utils=new Utils();
			String patterns="dd-MM-yyyy";											
			query="select d1.rootprocessinstanceid as codigo, d1.processname as nombreproceso,d1.description as descripcionproceso , d1.lastupdate as lastupdate, "
									+ "d1.var_value as cedula, d2.var_value as nombre, d3.var_value as apellido, "
									+ "d4.var_value as tipoinstitucion, d5.var_value as institucionsolicitante, "
									+ "d6.var_value as fechasolicitud, d7.var_value as genero, "
									+ "d8.var_value as motivo, d9.var_value as nomina "
									+ "from arch_process_instance_lastupdate d1, "
									+ "arch_process_instance_lastupdate d2, arch_process_instance_lastupdate d3,"
									+ " arch_process_instance_lastupdate d4, arch_process_instance_lastupdate d5, "
									+ "arch_process_instance_lastupdate d6, arch_process_instance_lastupdate d7, "
									+ "arch_process_instance_lastupdate d8, arch_process_instance_lastupdate d9 where "
									+ "d1.rootprocessinstanceid=d2.rootprocessinstanceid and"
									+ " d1.rootprocessinstanceid=d3.rootprocessinstanceid and  "
									+ "d1.rootprocessinstanceid=d4.rootprocessinstanceid and  "
									+ "d1.rootprocessinstanceid=d5.rootprocessinstanceid and  "
									+ "d1.rootprocessinstanceid=d6.rootprocessinstanceid and  "
									+ "d1.rootprocessinstanceid=d7.rootprocessinstanceid and  "
									+ "d1.rootprocessinstanceid=d8.rootprocessinstanceid and "
									+ " d1.rootprocessinstanceid=d9.rootprocessinstanceid and "
									+ "d1.name='a_cedula' and d2.name='b_nombre' and d3.name='c_apellido' and "
									+ "d4.name='e1_tipoInstitucion' and d5.name='e_institucionSolictante' and "
									+ "d6.name='d_fechaSolicitud' and d7.name='h_genero' and d8.name='ee_motivo' "
									+ "and  d9.name='j_nomina' ";
							
							
				if(parameters.get("cedula")!=null){
					query+=" and d1.var_value=?";								
					ps = cnn.prepareStatement(query);
					ps.setString(1,parameters.get("cedula"));
													
				}else if(parameters.get("codigo")!=null){
					query+=" and d1.rootprocessinstanceid=?";
					ps = cnn.prepareStatement(query);
					ps.setLong(1,Long.valueOf(parameters.get("codigo")));
								
				}
							
				if(parameters.get("cedula")==null&&parameters.get("codigo")==null){
					int i=0;
					if(parameters.get("tipoinstitucion")!=null){
						query+=" and d4.var_value=? ";								
					}								
					if(parameters.get("institucionsolicitante")!=null){
						query+=" and d5.var_value=? ";																
					}								
					if(parameters.get("nomina")!=null){
						query+=" and d9.var_value=? ";
					}								
					if(parameters.get("genero")!=null){
						query+=" and d7.var_value=? ";
					}								
					if(parameters.get("motivo")!=null){
						query+=" and d8.var_value=? ";
					}
					if(parameters.get("fechasolicitudinicial")!=null&&parameters.get("fechasolicitudfinal")!=null){
						//query+=" and (d6.dates>=to_timestamp(?,'YYYY-MM-DD HH24:MI:SS') and d6.dates <= to_timestamp(?,'YYYY-MM-DD HH24:MI:SS'))";
						query+=" and (d6.dates>=to_date(?,'YYYY-MM-DD') and d6.dates<= to_date(?,'YYYY-MM-DD'))";
					}		
								
					ps = cnn.prepareStatement(query);
								
					if(parameters.get("tipoinstitucion")!=null){
							ps.setString(++i,parameters.get("tipoinstitucion"));																
					}								
					if(parameters.get("institucionsolicitante")!=null){				
						ps.setString(++i,parameters.get("institucionsolicitante"));								
					}								
					if(parameters.get("nomina")!=null){				
						ps.setString(++i,parameters.get("nomina"));									
					}								
					if(parameters.get("genero")!=null){
						ps.setString(++i,parameters.get("genero"));																
					}								
					if(parameters.get("motivo")!=null){
						ps.setString(++i,parameters.get("motivo"));									
					}
					//SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd",Locale.ENGLISH);
								 
					if(parameters.get("fechasolicitudinicial")!=null&&parameters.get("fechasolicitudfinal")!=null){
						//System.out.println("fecha inicial "+(formatter.parse(parameters.get("fechasolicitudinicial"))));
						//System.out.println("fecha final "+(formatter.parse(parameters.get("fechasolicitudfinal"))));
						ps.setString(++i,parameters.get("fechasolicitudinicial"));
						ps.setString(++i,parameters.get("fechasolicitudfinal"));															
									
					}
								
			}
						  
			rs = ps.executeQuery();
			String fechaSolicitud;
			String fechaActualizacion;
			while(rs.next()){
							
				fechaSolicitud=utils.getSympleFormattedDateUS(patterns,rs.getString("fechasolicitud").
								replace("<?xml version=\"1.0\" ?><date>", "").replace("</date>", "").trim());
				fechaActualizacion=utils.getFormattedDate(patterns,(new Date(rs.getLong("lastupdate")).toString()));
				
				consultaGenerica=new ConsultaGenerica(String.valueOf(rs.getLong("codigo")),rs.getString("cedula")
										,rs.getString("nombreproceso"),rs.getString("nombre"),rs.getString("apellido"),
										fechaSolicitud,fechaActualizacion);							
				data.add(consultaGenerica);
								
								
			}
				
	  } catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		this.CloseConnetion();
	  }
		
      return data;
		
	}	
    /**
     * extrae informacion de los procesos en curso
     * @param cedula
     * @return data
     */ 
	public List<ConsultaGenerica> executeQueryOnlineHead(String cedula){
		
		List<ConsultaGenerica> data=new ArrayList<ConsultaGenerica>();
		ConsultaGenerica consultaGenerica=null;	
		try{
			String query=null;				
			Utils utils=new Utils();
			String patterns="dd-MM-yyyy";
			query="select d1.id as codigo,d2.name as name,concat(cast(d2.intvalue as varchar),"
				    +"cast(d2.shorttextvalue as varchar),cast(d2.longvalue as varchar),"
					+"cast(d2.booleanvalue as varchar),cast(d2.doublevalue as varchar),"
					+"cast(d2.floatvalue as varchar),cast(d2.clobvalue as varchar),"
					+"cast(d2.blobvalue as varchar)) as cedula, d2.classname as tipo,"
					+"d1.lastupdate as lastupdate,d1.name as nombreproceso,"
					+"d1.description as descripcion,concat(cast(d5.intvalue as varchar),cast(d5.shorttextvalue as " 
					+"varchar),cast(d5.longvalue as varchar),cast(d5.booleanvalue as " 
					+"varchar),cast(d5.doublevalue as varchar),cast(d5.floatvalue as " 
					+"varchar),cast(d5.clobvalue as varchar),cast(d5.blobvalue as varchar)"
					+") as fechasolicitud, concat(cast(d3.intvalue as varchar),cast(d3.shorttextvalue as " 
					+"varchar),cast(d3.longvalue as varchar),cast(d3.booleanvalue as " 
					+"varchar),cast(d3.doublevalue as varchar),cast(d3.floatvalue as "
					+"varchar),cast(d3.clobvalue as varchar),cast(d2.blobvalue as varchar)"
					+") as nombre,concat(cast(d4.intvalue as varchar),"
					+"cast(d4.shorttextvalue as varchar),cast(d4.longvalue as varchar),"
					+"cast(d4.booleanvalue as varchar),cast(d4.doublevalue as varchar),"
					+"cast(d4.floatvalue as varchar),cast(d4.clobvalue as varchar),"
					+"cast(d4.blobvalue as varchar)) as " 
					+"apellido from process_instance d1,data_instance d2,"
					+"data_instance d3,data_instance d4,data_instance d5 where d1.id=d2.containerid and " 
					+"d1.id=d3.containerid and d1.id=d4.containerid and d1.id=d5.containerid and (d2.name='a_cedula' "
					+"and d2.intvalue=?) and d3.name='b_nombre'and d4.name='c_apellido' and d5.name='d_fechaSolicitud' order by d1.id";			
					ps = cnn.prepareStatement(query);
					ps.setInt(1,Integer.valueOf(cedula));
					rs = ps.executeQuery();
					String fechaSolicitud;
					String fechaActualizacion;
					while(rs.next()){
						fechaSolicitud=utils.getSympleFormattedDateUS(patterns,rs.getString("fechasolicitud").
								 replace("<?xml version=\"1.0\" ?><date>", "").replace("</date>", "").trim());
						fechaActualizacion=utils.getFormattedDate(patterns,(new Date(rs.getLong("lastupdate")).toString()));
						System.out.println(rs.getString("fechasolicitud").
								 replace("<?xml version=\"1.0\" ?><date>", "").replace("</date>", "").trim());
						consultaGenerica=new ConsultaGenerica(String.valueOf(rs.getLong("codigo")),rs.getString("cedula")
								,rs.getString("nombreproceso"),rs.getString("nombre"),rs.getString("apellido"),
								fechaSolicitud,fechaActualizacion);							
					    data.add(consultaGenerica);		
						
					}
		
		}catch(Exception e){
			e.printStackTrace();
		}
		return data;
	}
	
	
    /**
     * se extrae informacion de las variables de los procesos finalizados
     * @param caseId
     * @return data
     */ 
	public List<Map<String,String>> executeQueryVariableHistory(String caseId){	
		 Map<String,String> data=null;
		 List<Map<String,String>> listaData=new ArrayList<Map<String,String>>();
		 String name=null;
		 String var_value=null;
		 try{
				String query=null;			
				Utils utils=new Utils();
				String patterns="EEE MMM dd HH:mm:ss z yyyy";	
				query="select rootprocessinstanceid as codigo, name,"
			  		+ "var_value, classname, processname, lastupdate from arch_process_instance_lastupdate where rootprocessinstanceid=?";													
				ps = cnn.prepareStatement(query);
				ps.setInt(1,Integer.valueOf(caseId));
				rs = ps.executeQuery();				
								
				while(rs.next()){
					
					data=new HashMap<String,String>();
					data.put("id",caseId);
					data.put("processname",rs.getString("processname"));					
	     			name=rs.getString("name");	
				    if(!rs.getString("classname").equals("java.util.Date"))
					     var_value=rs.getString("var_value");
					 else										 
						 var_value= utils.getSympleFormattedDateUS(patterns,rs.getString("var_value").
						 replace("<?xml version=\"1.0\" ?><date>", "").replace("</date>", "").trim());								
	     			 data.put("name",name);
	     			 data.put("value",var_value);
	     			 data.put("type",rs.getString("classname"));
	     			listaData.add(data);     			 
					
				}
		  }catch(Exception e){
			  e.printStackTrace();
		  }
		return listaData;
	}

    /**
     * cierra la conexion a la base de datos
     */ 
	public void CloseConnetion(){
		
		try {
			cnn.close();
			ps.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
    
	

}
