package ve.gob.cnti.gestion.utils;

import java.io.*;
import java.net.URL;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;

import org.apache.commons.io.IOUtils;
import org.xhtmlrenderer.pdf.ITextRenderer;

import ve.gob.cnti.gestion.alfresco.ICmisController;
import ve.gob.cnti.gestion.alfresco.impl.AlfrescoLoadFiles;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

@ManagedBean(name="pdfCreator", eager = true)
@RequestScoped
public class PdfCreator {
	
	private final String TEMP_FILE = System.getProperty("java.io.tmpdir");
	
	public void crearActuacion() throws IOException{
		
		//Creando contenido de prueba
    	String contenido_test = "";
    	for(int i=0;i<100;i++){
    		contenido_test+=i+") Esto es un test de prueba.<br/>";
    	}
		
		createPDF("/views/interfazportalciudadano/test/pdfTest/sources/plantilla/","asdr", "Mi test",contenido_test);
	}
	
	/**Método que genera un pdf a partir de la actuación sobre una tarea.
	 * 
	 * @param pathPlantilla path relativo en donde se encuentra la plantilla para generar la actuación.
	 * @param pathAlfresco path en alfresco en donde se guardara el PDF de la actuación generado.
	 * @param namePDF Nombre del PDF con el que se guardara la actuación generada.
	 * @param content Contenido de la actuación.
	 * 
	 * @return {@link String} <b>Id</b> en alfresco en donde se guardo la actuación.
	 * @throws IOException
	 * @author Joaquin Pereira y Alexis Veliz 
	 */
	public String createPDF(String pathPlantilla,String pathAlfresco,String namePDF, String content)throws IOException{
		String id = "";
	    try {
	    	//Obteniendo Contenido de la plantilla y obteniendo la ruta absoluta del directorio de la plantilla.
	    	String htmlcontent="";
	    	String path = getRealPath(pathPlantilla);	    	
	    	
	    	FileInputStream inputStream = new FileInputStream(path+"view.xhtml");
            htmlcontent = IOUtils.toString(inputStream);
            inputStream.close();
	        
            //Parseando y LLenando el HTML con los datos de la actuación.
	    	Document doc = Jsoup.parse(htmlcontent, "UTF-8");
	    	
	    	//Guardando el contenido de la actuación en el html
			doc.getElementById("content").html(content);
			
			//Obteniendo las imagenes de la plantilla para inyectar el path absoluto.
			Elements media = doc.select("[src]");
			int i = 1;
	        for (Element src : media) {
	            if (src.tagName().equals("img")){
	                String src2 = src.attr("src");
	                doc.getElementById("imagen_"+i).removeAttr("src");	                
	                doc.getElementById("imagen_"+i).attr("src", path+src2);	                
	            }
	            i++;
	        }	
	        
	        //Inyectando la ruta Absoluta al css.
	        doc.getElementById("cssMain").removeAttr("href");
	        doc.getElementById("cssMain").attr("href", path+"view.css");

	        //Inyectando la la ruta Absoluta a la imagen de la cabecera
	        String style[] = doc.select("style").first().data().split("'");
	        doc.select("style").first().html(style[0]+"'"+path+style[1]+"'"+style[2]);
			
			//Operaciones de Creación del PDF con la libreria ITEX
			OutputStream os = null;			
			File fileTemp = new File(TEMP_FILE+"/"+namePDF+".pdf");		
			os = new FileOutputStream(fileTemp);
			ITextRenderer renderer = new ITextRenderer();
			renderer.setDocumentFromString(doc.html());
			renderer.layout();
			renderer.createPDF(os);
			os.close();
			
			//Subida de PDF Creado a alfresco
			ICmisController iCmisController = new AlfrescoLoadFiles();  
			id = iCmisController.uploadDocument(fileTemp, pathAlfresco,fileTemp.getName());
			
			//Borrando Archivo Temporal del Sistema.
			fileTemp.delete();
			return id;
    	} catch (Exception e) {
    		System.out.printf("Error", e);
    	}
		return id;
	}
	
	/**Obtiene la ruta absoluta en File System de una vista o ruta de directorios. Dichas rutas son referentes a las vista.
	 * @author Joaquin Pereira.
	 * @param path Ruta relativa de un directorio o vista. 
	 * @return Ruta absoluta en File System de una vista o ruta de directorios
	 */
	public String getRealPath(String path){		
		URL rutaClass = PdfCreator.class.getProtectionDomain().getCodeSource().getLocation();
		String ruta = rutaClass.toString();		
		ruta=ruta.split("/WEB-INF/")[0];
		ruta=ruta.split(":")[1];
		System.out.println(ruta);
		ruta+=path;		
		return ruta;
	}
	
	public void mensaje(String msj,String var){
		System.out.println("*********************************************************************************");
		System.out.println(msj+" ====>> "+var);
		System.out.println("*********************************************************************************\n");
	}
}
